@@ -1,20 +1,22 @@
 use chrono::{NaiveDateTime, TimeDelta, Utc};
+use defguard_common::{
+    VERSION,
+    config::server_config,
+    db::{Id, models::Settings},
+    random::gen_alphanumeric,
+};
+use defguard_mail::{
+    Mail,
+    templates::{self, TemplateError, safe_tera},
+};
 use reqwest::Url;
-use sqlx::{query, query_as, Error as SqlxError, PgConnection, PgExecutor, PgPool};
-use tera::{Context, Tera};
+use sqlx::{Error as SqlxError, PgConnection, PgExecutor, PgPool, query, query_as};
+use tera::Context;
 use thiserror::Error;
 use tokio::sync::mpsc::UnboundedSender;
 use tonic::{Code, Status};
 
-use super::{settings::Settings, User};
-use crate::{
-    db::Id,
-    mail::Mail,
-    random::gen_alphanumeric,
-    server_config,
-    templates::{self, TemplateError},
-    VERSION,
-};
+use super::User;
 
 pub static ENROLLMENT_TOKEN_TYPE: &str = "ENROLLMENT";
 pub static PASSWORD_RESET_TOKEN_TYPE: &str = "PASSWORD_RESET";
@@ -317,7 +319,7 @@ impl Token {
     /// - admin_last_name
     /// - admin_email
     /// - admin_phone
-    pub async fn get_welcome_message_context(
+    async fn get_welcome_message_context(
         &self,
         transaction: &mut PgConnection,
     ) -> Result<Context, TokenError> {
@@ -355,8 +357,8 @@ impl Token {
         let settings = Settings::get_current_settings();
 
         // load configured content as template
-        let mut tera = Tera::default();
-        tera.add_raw_template("welcome_page", &settings.enrollment_welcome_message()?)?;
+        let mut tera = safe_tera();
+        tera.add_raw_template("welcome_page", &enrollment_welcome_message(&settings)?)?;
 
         let context = self.get_welcome_message_context(&mut *transaction).await?;
 
@@ -373,8 +375,8 @@ impl Token {
         let settings = Settings::get_current_settings();
 
         // load configured content as template
-        let mut tera = Tera::default();
-        tera.add_raw_template("welcome_email", &settings.enrollment_welcome_email()?)?;
+        let mut tera = safe_tera();
+        tera.add_raw_template("welcome_email", &enrollment_welcome_email(&settings)?)?;
 
         let context = self.get_welcome_message_context(&mut *transaction).await?;
         let content = tera.render("welcome_email", &context)?;
@@ -616,21 +618,19 @@ impl User<Id> {
     }
 }
 
-impl Settings {
-    pub fn enrollment_welcome_message(&self) -> Result<String, TokenError> {
-        self.enrollment_welcome_message.clone().ok_or_else(|| {
-            error!("Enrollment welcome message not configured");
-            TokenError::WelcomeMsgNotConfigured
-        })
-    }
+pub fn enrollment_welcome_message(settings: &Settings) -> Result<String, TokenError> {
+    settings.enrollment_welcome_message.clone().ok_or_else(|| {
+        error!("Enrollment welcome message not configured");
+        TokenError::WelcomeMsgNotConfigured
+    })
+}
 
-    pub fn enrollment_welcome_email(&self) -> Result<String, TokenError> {
-        if self.enrollment_use_welcome_message_as_email {
-            return self.enrollment_welcome_message();
-        }
-        self.enrollment_welcome_email.clone().ok_or_else(|| {
-            error!("Enrollment welcome email not configured");
-            TokenError::WelcomeEmailNotConfigured
-        })
+pub fn enrollment_welcome_email(settings: &Settings) -> Result<String, TokenError> {
+    if settings.enrollment_use_welcome_message_as_email {
+        return enrollment_welcome_message(settings);
     }
+    settings.enrollment_welcome_email.clone().ok_or_else(|| {
+        error!("Enrollment welcome email not configured");
+        TokenError::WelcomeEmailNotConfigured
+    })
 }
