@@ -1,7 +1,7 @@
 use model_derive::Model;
-use sqlx::{query_as, Error as SqlxError, PgPool};
+use sqlx::{Error as SqlxError, PgPool, query_as};
 
-use crate::db::{Id, NoId};
+use defguard_common::db::{Id, NoId};
 
 #[derive(Model)]
 pub struct OAuth2AuthorizedApp<I = NoId> {
