@@ -1,82 +1,49 @@
 use std::{collections::HashSet, fmt, time::SystemTime};
 
 use argon2::{
-    password_hash::{
-        errors::Error as HashError, rand_core::OsRng, PasswordHash, PasswordHasher,
-        PasswordVerifier, SaltString,
-    },
     Argon2,
+    password_hash::{
+        PasswordHash, PasswordHasher, PasswordVerifier, SaltString, errors::Error as HashError,
+        rand_core::OsRng,
+    },
 };
 use axum::http::StatusCode;
+use defguard_mail::templates::UserContext;
 use model_derive::Model;
 #[cfg(test)]
 use rand::{
+    Rng,
     distributions::{Alphanumeric, DistString, Standard},
     prelude::Distribution,
-    Rng,
 };
+use serde::Serialize;
 use sqlx::{
-    query, query_as, query_scalar, Error as SqlxError, FromRow, PgConnection, PgExecutor, PgPool,
-    Type,
+    Error as SqlxError, FromRow, PgConnection, PgExecutor, PgPool, query, query_as, query_scalar,
 };
 use tokio::sync::broadcast::Sender;
-use totp_lite::{totp_custom, Sha1};
-use utoipa::ToSchema;
+use totp_lite::{Sha1, totp_custom};
 
 use super::{
+    MFAInfo, OAuth2AuthorizedAppInfo, SecurityKey,
     device::{Device, DeviceInfo, DeviceType, UserDevice},
     group::Group,
     webauthn::WebAuthn,
-    MFAInfo, OAuth2AuthorizedAppInfo, SecurityKey,
 };
 use crate::{
     auth::{EMAIL_CODE_DIGITS, TOTP_CODE_DIGITS, TOTP_CODE_VALIDITY_PERIOD},
-    db::{models::group::Permission, GatewayEvent, Id, NoId, Session, WireguardNetwork},
+    db::{GatewayEvent, Session, WireguardNetwork, models::group::Permission},
     enterprise::limits::update_counts,
     error::WebError,
-    grpc::{
-        gateway::{send_multiple_wireguard_events, send_wireguard_event},
-        proto::proxy::MfaMethod,
-    },
+    grpc::gateway::{send_multiple_wireguard_events, send_wireguard_event},
+};
+use defguard_common::{
+    config::server_config,
+    db::{Id, NoId, models::MFAMethod},
     random::{gen_alphanumeric, gen_totp_secret},
-    server_config,
 };
 
 const RECOVERY_CODES_COUNT: usize = 8;
 
-#[derive(Clone, Debug, Deserialize, Serialize, PartialEq, Eq, Hash, ToSchema, Type)]
-#[sqlx(type_name = "mfa_method", rename_all = "snake_case")]
-pub enum MFAMethod {
-    None,
-    OneTimePassword,
-    Webauthn,
-    Email,
-}
-
-impl From<MfaMethod> for MFAMethod {
-    fn from(method: MfaMethod) -> Self {
-        match method {
-            MfaMethod::Totp => Self::OneTimePassword,
-            MfaMethod::Email => Self::Email,
-        }
-    }
-}
-
-impl fmt::Display for MFAMethod {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(
-            f,
-            "{}",
-            match self {
-                MFAMethod::None => "None",
-                MFAMethod::OneTimePassword => "TOTP",
-                MFAMethod::Webauthn => "WebAuthn",
-                MFAMethod::Email => "Email",
-            }
-        )
-    }
-}
-
 // User information ready to be sent as part of diagnostic data.
 #[derive(Serialize)]
 pub struct UserDiagnostic {
@@ -192,6 +159,15 @@ fn hash_password(password: &str) -> Result<String, HashError> {
         .to_string())
 }
 
+impl From<User<Id>> for UserContext {
+    fn from(value: User<Id>) -> Self {
+        Self {
+            last_name: value.last_name,
+            first_name: value.first_name,
+        }
+    }
+}
+
 impl User {
     #[must_use]
     pub fn new<S: Into<String>>(
@@ -265,7 +241,7 @@ impl<I> User<I> {
     /// We assume the user is enrolled if they have a password set
     /// or they have logged in using an external OIDC.
     #[must_use]
-    pub(crate) fn is_enrolled(&self) -> bool {
+    pub fn is_enrolled(&self) -> bool {
         self.password_hash.is_some() || self.openid_sub.is_some() || self.from_ldap
     }
 
@@ -369,7 +345,7 @@ impl User<Id> {
         .await
     }
 
-    /// Verify the state of mfa flags are correct.
+    /// Verify the state of MFA flags are correct.
     /// Recovers from invalid mfa_method
     /// Use this function after removing any of the authentication factors.
     pub async fn verify_mfa_state(&mut self, pool: &PgPool) -> Result<(), WebError> {
@@ -501,7 +477,9 @@ impl User<Id> {
         for location_id in affected_location_ids {
             if let Some(location) = WireguardNetwork::find_by_id(&mut *conn, location_id).await? {
                 if let Some(firewall_config) = location.try_get_firewall_config(&mut *conn).await? {
-                    debug!("Sending firewall config update for location {location} affected by deleting user {username} devices");
+                    debug!(
+                        "Sending firewall config update for location {location} affected by deleting user {username} devices"
+                    );
                     events.push(GatewayEvent::FirewallConfigChanged(
                         location_id,
                         firewall_config,
@@ -845,9 +823,9 @@ impl User<Id> {
     {
         query_as!(
             Self,
-            "SELECT id, username, password_hash, last_name, first_name, email, \
-            phone, mfa_enabled, totp_enabled, email_mfa_enabled, \
-            totp_secret, email_mfa_secret, mfa_method \"mfa_method: _\", recovery_codes, is_active, openid_sub, \
+            "SELECT id, username, password_hash, last_name, first_name, email, phone, mfa_enabled, \
+            totp_enabled, email_mfa_enabled, totp_secret, email_mfa_secret, \
+            mfa_method \"mfa_method: _\", recovery_codes, is_active, openid_sub, \
             from_ldap, ldap_pass_randomized, ldap_rdn, ldap_user_path \
             FROM \"user\" WHERE username = $1",
             username
@@ -865,9 +843,10 @@ impl User<Id> {
     {
         query_as!(
             Self,
-            "SELECT id, username, password_hash, last_name, first_name, email, phone, \
-            mfa_enabled, totp_enabled, email_mfa_enabled, totp_secret, email_mfa_secret, \
-            mfa_method \"mfa_method: _\", recovery_codes, is_active, openid_sub, from_ldap, ldap_pass_randomized, ldap_rdn, ldap_user_path \
+            "SELECT id, username, password_hash, last_name, first_name, email, phone, mfa_enabled, \
+            totp_enabled, email_mfa_enabled, totp_secret, email_mfa_secret, \
+            mfa_method \"mfa_method: _\", recovery_codes, is_active, openid_sub, from_ldap, \
+            ldap_pass_randomized, ldap_rdn, ldap_user_path \
             FROM \"user\" WHERE email ILIKE $1",
             email
         )
@@ -875,6 +854,22 @@ impl User<Id> {
         .await
     }
 
+    /// Attempts to find user by username and then by email, if none is initially found.
+    pub async fn find_by_username_or_email(
+        conn: &mut PgConnection,
+        username_or_email: &str,
+    ) -> Result<Option<Self>, SqlxError> {
+        let maybe_user = Self::find_by_username(&mut *conn, username_or_email).await?;
+        if let Some(user) = maybe_user {
+            Ok(Some(user))
+        } else {
+            debug!(
+                "Failed to find user by username {username_or_email}. Attempting to find by email"
+            );
+            Ok(Self::find_by_email(&mut *conn, username_or_email).await?)
+        }
+    }
+
     pub(crate) async fn find_many_by_emails<'e, E>(
         executor: E,
         emails: &[&str],
@@ -893,7 +888,6 @@ impl User<Id> {
         .await
     }
 
-    // FIXME: Remove `LIMIT 1` when `openid_sub` is unique.
     pub(crate) async fn find_by_sub<'e, E>(
         executor: E,
         sub: &str,
@@ -907,7 +901,7 @@ impl User<Id> {
             mfa_enabled, totp_enabled, email_mfa_enabled, totp_secret, email_mfa_secret, \
             mfa_method \"mfa_method: _\", recovery_codes, is_active, openid_sub, \
             from_ldap, ldap_pass_randomized, ldap_rdn, ldap_user_path \
-            FROM \"user\" WHERE openid_sub = $1 LIMIT 1",
+            FROM \"user\" WHERE openid_sub = $1",
             sub
         )
         .fetch_optional(executor)
@@ -1201,28 +1195,28 @@ impl User<Id> {
 impl Distribution<User<Id>> for Standard {
     fn sample<R: Rng + ?Sized>(&self, rng: &mut R) -> User<Id> {
         User {
-            id: rng.gen(),
+            id: rng.r#gen(),
             username: Alphanumeric.sample_string(rng, 8),
             password_hash: rng
-                .gen::<bool>()
+                .r#gen::<bool>()
                 .then_some(Alphanumeric.sample_string(rng, 8)),
             last_name: Alphanumeric.sample_string(rng, 8),
             first_name: Alphanumeric.sample_string(rng, 8),
             email: format!("{}@defguard.net", Alphanumeric.sample_string(rng, 6)),
             // FIXME: generate an actual phone number
             phone: rng
-                .gen::<bool>()
+                .r#gen::<bool>()
                 .then_some(Alphanumeric.sample_string(rng, 9)),
-            mfa_enabled: rng.gen(),
+            mfa_enabled: rng.r#gen(),
             is_active: true,
             openid_sub: rng
-                .gen::<bool>()
+                .r#gen::<bool>()
                 .then_some(Alphanumeric.sample_string(rng, 8)),
-            totp_enabled: rng.gen(),
-            email_mfa_enabled: rng.gen(),
-            totp_secret: (0..20).map(|_| rng.gen()).collect(),
-            email_mfa_secret: (0..20).map(|_| rng.gen()).collect(),
-            mfa_method: match rng.gen_range(0..4) {
+            totp_enabled: rng.r#gen(),
+            email_mfa_enabled: rng.r#gen(),
+            totp_secret: (0..20).map(|_| rng.r#gen()).collect(),
+            email_mfa_secret: (0..20).map(|_| rng.r#gen()).collect(),
+            mfa_method: match rng.r#gen_range(0..4) {
                 0 => MFAMethod::None,
                 1 => MFAMethod::Webauthn,
                 2 => MFAMethod::OneTimePassword,
@@ -1244,25 +1238,25 @@ impl Distribution<User<NoId>> for Standard {
             id: NoId,
             username: Alphanumeric.sample_string(rng, 8),
             password_hash: rng
-                .gen::<bool>()
+                .r#gen::<bool>()
                 .then_some(Alphanumeric.sample_string(rng, 8)),
             last_name: Alphanumeric.sample_string(rng, 8),
             first_name: Alphanumeric.sample_string(rng, 8),
             email: format!("{}@defguard.net", Alphanumeric.sample_string(rng, 6)),
             // FIXME: generate an actual phone number
             phone: rng
-                .gen::<bool>()
+                .r#gen::<bool>()
                 .then_some(Alphanumeric.sample_string(rng, 9)),
-            mfa_enabled: rng.gen(),
+            mfa_enabled: rng.r#gen(),
             is_active: true,
             openid_sub: rng
-                .gen::<bool>()
+                .r#gen::<bool>()
                 .then_some(Alphanumeric.sample_string(rng, 8)),
-            totp_enabled: rng.gen(),
-            email_mfa_enabled: rng.gen(),
-            totp_secret: (0..20).map(|_| rng.gen()).collect(),
-            email_mfa_secret: (0..20).map(|_| rng.gen()).collect(),
-            mfa_method: match rng.gen_range(0..4) {
+            totp_enabled: rng.r#gen(),
+            email_mfa_enabled: rng.r#gen(),
+            totp_secret: (0..20).map(|_| rng.r#gen()).collect(),
+            email_mfa_secret: (0..20).map(|_| rng.r#gen()).collect(),
+            mfa_method: match rng.r#gen_range(0..4) {
                 0 => MFAMethod::None,
                 1 => MFAMethod::Webauthn,
                 2 => MFAMethod::OneTimePassword,
@@ -1279,14 +1273,14 @@ impl Distribution<User<NoId>> for Standard {
 
 #[cfg(test)]
 mod test {
+    use defguard_common::{
+        config::{DefGuardConfig, SERVER_CONFIG},
+        db::setup_pool,
+    };
     use sqlx::postgres::{PgConnectOptions, PgPoolOptions};
 
     use super::*;
-    use crate::{
-        config::DefGuardConfig,
-        db::{models::settings::initialize_current_settings, setup_pool},
-        SERVER_CONFIG,
-    };
+    use defguard_common::db::models::settings::initialize_current_settings;
 
     #[sqlx::test]
     async fn test_mfa_code(_: PgPoolOptions, options: PgConnectOptions) {
@@ -1410,10 +1404,12 @@ mod test {
 
         let mut user = fetched_user.unwrap();
         assert_eq!(user.recovery_codes.len(), RECOVERY_CODES_COUNT);
-        assert!(!user
-            .verify_recovery_code(&pool, "invalid code")
-            .await
-            .unwrap());
+        assert!(
+            !user
+                .verify_recovery_code(&pool, "invalid code")
+                .await
+                .unwrap()
+        );
         let codes = user.recovery_codes.clone();
         for code in &codes {
             assert!(user.verify_recovery_code(&pool, code).await.unwrap());
