@@ -6,22 +6,23 @@ use std::{
 };
 
 use chrono::NaiveDateTime;
+use defguard_common::db::{Id, NoId};
 use ipnetwork::{IpNetwork, IpNetworkError};
 use model_derive::Model;
 use sqlx::{
-    error::ErrorKind, postgres::types::PgRange, query, query_as, query_scalar, Error as SqlxError,
-    FromRow, PgConnection, PgExecutor, PgPool, Type,
+    Error as SqlxError, FromRow, PgConnection, PgExecutor, PgPool, Type, error::ErrorKind,
+    postgres::types::PgRange, query, query_as, query_scalar,
 };
 use thiserror::Error;
 
 use crate::{
+    DeviceType,
     appstate::AppState,
-    db::{Device, GatewayEvent, Group, Id, NoId, User, WireguardNetwork},
+    db::{Device, GatewayEvent, Group, User, WireguardNetwork, models::wireguard::LocationMfaMode},
     enterprise::{
         firewall::FirewallError,
         handlers::acl::{ApiAclAlias, ApiAclRule, EditAclAlias, EditAclRule},
     },
-    DeviceType,
 };
 
 #[derive(Debug, Error)]
@@ -567,7 +568,9 @@ pub fn parse_ports(ports: &str) -> Result<Vec<PortRange>, AclError> {
 fn map_relation_error(err: SqlxError, class: &str, id: Id) -> AclError {
     if let SqlxError::Database(dberror) = &err {
         if dberror.kind() == ErrorKind::ForeignKeyViolation {
-            error!("Failed to create ACL related object, foreign key violation: {class}({id}): {dberror}");
+            error!(
+                "Failed to create ACL related object, foreign key violation: {class}({id}): {dberror}"
+            );
             return AclError::InvalidRelationError(format!("{class}({id})"));
         }
     }
@@ -665,7 +668,9 @@ impl AclRule<Id> {
         .fetch_all(&mut *transaction)
         .await?;
         if !invalid_alias_ids.is_empty() {
-            error!("Cannot use aliases which have not been applied in an ACL rule. Invalid aliases: {invalid_alias_ids:?}");
+            error!(
+                "Cannot use aliases which have not been applied in an ACL rule. Invalid aliases: {invalid_alias_ids:?}"
+            );
             return Err(AclError::CannotUseModifiedAliasInRuleError(
                 invalid_alias_ids,
             ));
@@ -813,7 +818,7 @@ impl TryFrom<EditAclRule> for AclRule<NoId> {
                 .collect(),
             id: NoId,
             parent_id: None,
-            state: Default::default(),
+            state: RuleState::default(),
             name: rule.name,
             allow_all_users: rule.allow_all_users,
             deny_all_users: rule.deny_all_users,
@@ -900,8 +905,8 @@ impl AclRule<Id> {
             query_as!(
                 WireguardNetwork,
                 "SELECT n.id, name, address, port, pubkey, prvkey, endpoint, dns, allowed_ips, \
-                connected_at, mfa_enabled, keepalive_interval, peer_disconnect_threshold, \
-                acl_enabled, acl_default_allow \
+                connected_at, keepalive_interval, peer_disconnect_threshold, \
+                acl_enabled, acl_default_allow, location_mfa_mode \"location_mfa_mode: LocationMfaMode\" \
                 FROM aclrulenetwork r \
                 JOIN wireguard_network n \
                 ON n.id = r.network_id \
@@ -1596,7 +1601,9 @@ impl AclAlias {
         // check if any rules are using this alias
         let rules = existing_alias.get_rules(&mut *transaction).await?;
         if !rules.is_empty() {
-            error!("Deletion of alias ({id}) failed. Alias is currently used by following ACL rules: {rules:?}");
+            error!(
+                "Deletion of alias ({id}) failed. Alias is currently used by following ACL rules: {rules:?}"
+            );
             return Err(AclError::AliasUsedByRulesError(id));
         }
 
