@@ -1,10 +1,11 @@
 use std::ops::Bound;
 
-use rand::{thread_rng, Rng};
+use defguard_common::db::setup_pool;
+use rand::{Rng, thread_rng};
 use sqlx::postgres::{PgConnectOptions, PgPoolOptions};
 
 use super::*;
-use crate::{db::setup_pool, handlers::wireguard::parse_address_list};
+use crate::handlers::wireguard::parse_address_list;
 
 #[sqlx::test]
 async fn test_alias(_: PgPoolOptions, options: PgConnectOptions) {
@@ -177,13 +178,12 @@ async fn test_rule_relations(_: PgPoolOptions, options: PgConnectOptions) {
         "endpoint1".to_string(),
         None,
         Vec::new(),
-        false,
         100,
         100,
         false,
         false,
+        LocationMfaMode::Disabled,
     )
-    .unwrap()
     .save(&pool)
     .await
     .unwrap();
@@ -194,13 +194,12 @@ async fn test_rule_relations(_: PgPoolOptions, options: PgConnectOptions) {
         "endpoint2".to_string(),
         None,
         Vec::new(),
-        false,
         200,
         200,
         false,
         false,
+        LocationMfaMode::Disabled,
     )
-    .unwrap()
     .save(&pool)
     .await
     .unwrap();
@@ -472,14 +471,14 @@ async fn test_all_allowed_users(_: PgPoolOptions, options: PgConnectOptions) {
     let mut rng = thread_rng();
 
     // Create test users
-    let user_1: User<NoId> = rng.gen();
+    let user_1: User<NoId> = rng.r#gen();
     let user_1 = user_1.save(&pool).await.unwrap();
-    let user_2: User<NoId> = rng.gen();
+    let user_2: User<NoId> = rng.r#gen();
     let user_2 = user_2.save(&pool).await.unwrap();
-    let user_3: User<NoId> = rng.gen();
+    let user_3: User<NoId> = rng.r#gen();
     let user_3 = user_3.save(&pool).await.unwrap();
     // inactive user
-    let mut user_4: User<NoId> = rng.gen();
+    let mut user_4: User<NoId> = rng.r#gen();
     user_4.is_active = false;
     let user_4 = user_4.save(&pool).await.unwrap();
 
@@ -587,14 +586,14 @@ async fn test_all_denied_users(_: PgPoolOptions, options: PgConnectOptions) {
     let mut rng = thread_rng();
 
     // Create test users
-    let user_1: User<NoId> = rng.gen();
+    let user_1: User<NoId> = rng.r#gen();
     let user_1 = user_1.save(&pool).await.unwrap();
-    let user_2: User<NoId> = rng.gen();
+    let user_2: User<NoId> = rng.r#gen();
     let user_2 = user_2.save(&pool).await.unwrap();
-    let user_3: User<NoId> = rng.gen();
+    let user_3: User<NoId> = rng.r#gen();
     let user_3 = user_3.save(&pool).await.unwrap();
     // inactive user
-    let mut user_4: User<NoId> = rng.gen();
+    let mut user_4: User<NoId> = rng.r#gen();
     user_4.is_active = false;
     let user_4 = user_4.save(&pool).await.unwrap();
 
