@@ -1,17 +1,25 @@
-use axum::{extract::State, http::StatusCode, Json};
+use axum::{Json, extract::State, http::StatusCode};
 use axum_client_ip::InsecureClientIp;
 use axum_extra::{
+    TypedHeader,
     extract::{
-        cookie::{Cookie, SameSite},
         CookieJar, PrivateCookieJar,
+        cookie::{Cookie, SameSite},
     },
     headers::UserAgent,
-    TypedHeader,
+};
+use base64::{Engine, prelude::BASE64_STANDARD};
+use defguard_common::{
+    config::server_config,
+    db::{
+        Id,
+        models::{Settings, settings::OpenidUsernameHandling},
+    },
 };
 use openidconnect::{
-    core::{CoreAuthenticationFlow, CoreClient, CoreProviderMetadata, CoreUserInfoClaims},
     AuthorizationCode, ClientId, ClientSecret, CsrfToken, EndpointMaybeSet, EndpointNotSet,
     EndpointSet, IssuerUrl, Nonce, OAuth2TokenResponse, RedirectUrl, Scope,
+    core::{CoreAuthenticationFlow, CoreClient, CoreProviderMetadata, CoreUserInfoClaims},
 };
 use reqwest::Url;
 use serde_json::json;
@@ -21,11 +29,14 @@ use time::Duration;
 const COOKIE_MAX_AGE: Duration = Duration::days(1);
 static CSRF_COOKIE_NAME: &str = "csrf";
 static NONCE_COOKIE_NAME: &str = "nonce";
+// The select_account prompt is not supported by all providers, most notably not by JumpCloud.
+// Currently it's only enabled for Google, as it was tested to work there.
+pub(crate) const SELECT_ACCOUNT_SUPPORTED_PROVIDERS: &[&str] = &["Google"];
 
 use super::LicenseInfo;
 use crate::{
     appstate::AppState,
-    db::{models::settings::OpenidUsernameHandling, Id, Settings, User},
+    db::User,
     enterprise::{
         db::models::openid_provider::OpenIdProvider,
         directory_sync::sync_user_groups_if_configured, ldap::utils::ldap_update_user_state,
@@ -33,10 +44,10 @@ use crate::{
     },
     error::WebError,
     handlers::{
-        auth::create_session, user::check_username, ApiResponse, AuthResponse, SESSION_COOKIE_NAME,
-        SIGN_IN_COOKIE_NAME,
+        ApiResponse, AuthResponse, SESSION_COOKIE_NAME, SIGN_IN_COOKIE_NAME,
+        auth::create_session,
+        user::{MAX_USERNAME_CHARS, check_username},
     },
-    server_config,
 };
 
 /// Prune the given username from illegal characters in accordance with the following rules:
@@ -48,6 +59,7 @@ use crate::{
 /// - starts with non-special character
 /// - only special characters allowed: . - _
 /// - no whitespaces
+#[must_use]
 pub fn prune_username(username: &str, handling: OpenidUsernameHandling) -> String {
     let mut result = username.to_string();
 
@@ -79,18 +91,17 @@ pub fn prune_username(username: &str, handling: OpenidUsernameHandling) -> Strin
         }
     }
 
-    result.truncate(64);
-
+    result.truncate(MAX_USERNAME_CHARS);
     result
 }
 
 /// Create HTTP client and prevent following redirects
-async fn get_async_http_client() -> Result<reqwest::Client, WebError> {
+fn get_async_http_client() -> Result<reqwest::Client, WebError> {
     reqwest::Client::builder()
         .redirect(reqwest::redirect::Policy::none())
         .build()
         .map_err(|err| {
-            error!("Failed to create HTTP client: {err:?}");
+            error!("Failed to create HTTP client: {err}");
             WebError::Http(StatusCode::INTERNAL_SERVER_ERROR)
         })
 }
@@ -98,20 +109,48 @@ async fn get_async_http_client() -> Result<reqwest::Client, WebError> {
 async fn get_provider_metadata(url: &str) -> Result<CoreProviderMetadata, WebError> {
     let issuer_url = IssuerUrl::new(url.to_string()).map_err(|err| {
         WebError::BadRequest(format!(
-            "Failed to create issuer URL from the provided URL: {url}. Error details: {err:?}",
+            "Failed to create issuer URL from the provided URL: {url}. Error details: {err}",
         ))
     })?;
-    let async_http_client = get_async_http_client().await?;
-    // Discover the provider metadata based on a known base issuer URL
-    // The url should be in the form of e.g. https://accounts.google.com
-    // The url shouldn't contain a .well-known part, it will be added automatically
+    let async_http_client = get_async_http_client()?;
+    // Discover the provider metadata based on a known base issuer URL.
+    // The URL should be in the form of e.g. https://accounts.google.com.
+    // The URL shouldn't contain the ".well-known" part – it will be added automatically.
     match CoreProviderMetadata::discover_async(issuer_url, &async_http_client).await {
         Ok(provider_metadata) => Ok(provider_metadata),
-        Err(err) => {
-            Err(WebError::Authorization(format!(
-                "Failed to discover provider metadata, make sure the provider's URL is correct: {url}. Error details: {err:?}",
-            )))
+        Err(err) => Err(WebError::Authorization(format!(
+            "Failed to discover provider metadata, make sure the provider's URL is correct: {url}. \
+            Error details: {err}",
+        ))),
+    }
+}
+
+/// Build a state with optional embedded data. Useful for passing additional information around the
+/// authentication flow.
+pub(crate) fn build_state(state_data: Option<String>) -> CsrfToken {
+    let csrf_token = CsrfToken::new_random();
+    if let Some(data) = state_data {
+        let combined = format!("{}.{data}", csrf_token.secret());
+        let encoded = BASE64_STANDARD.encode(combined);
+        CsrfToken::new(encoded)
+    } else {
+        csrf_token
+    }
+}
+
+/// Extract the state data from the provided state.
+pub(crate) fn extract_state_data(state: &str) -> Option<String> {
+    let decoded = BASE64_STANDARD.decode(state).ok()?;
+    let decoded_str = String::from_utf8(decoded).ok()?;
+    let result = decoded_str.split_once('.');
+    if let Some((part1, part2)) = result {
+        if part1.is_empty() {
+            None
+        } else {
+            Some(part2.to_string())
         }
+    } else {
+        None
     }
 }
 
@@ -160,12 +199,12 @@ pub(crate) async fn user_from_claims(
         ));
     };
     let (client_id, core_client) = make_oidc_client(callback_url, &provider).await?;
-    let async_http_client = get_async_http_client().await?;
+    let async_http_client = get_async_http_client()?;
     // Exchange code for ID token.
     let token_response = match core_client
         .exchange_code(code)
         .map_err(|err| {
-            error!("Failed to exchange code for ID token: {err:?}");
+            error!("Failed to exchange code for ID token: {err}");
             WebError::Http(StatusCode::INTERNAL_SERVER_ERROR)
         })?
         .request_async(&async_http_client)
@@ -174,7 +213,7 @@ pub(crate) async fn user_from_claims(
         Ok(token) => token,
         Err(err) => {
             return Err(WebError::Authorization(format!(
-                "Failed to exchange code for ID token; OpenID provider error: {err:?}"
+                "Failed to exchange code for ID token; OpenID provider error: {err}"
             )));
         }
     };
@@ -212,7 +251,7 @@ pub(crate) async fn user_from_claims(
             client_id.as_str(),
             audiences
                 .iter()
-                .map(|aud| aud.to_string())
+                .map(|aud| aud.as_str())
                 .collect::<Vec<_>>()
                 .join(", ")
         )));
@@ -223,7 +262,7 @@ pub(crate) async fn user_from_claims(
             audiences
                 .iter()
                 .filter(|&aud| **aud != *client_id)
-                .map(|aud| aud.to_string())
+                .map(|aud| aud.as_str())
                 .collect::<Vec<_>>()
                 .join(", ")
         );
@@ -237,26 +276,6 @@ pub(crate) async fn user_from_claims(
             .to_string(),
     ))?;
 
-    // Try to get the username from the preferred_username claim.
-    // If it's not there, extract it from email.
-    let username = if let Some(username) = token_claims.preferred_username() {
-        debug!("Preferred username {username:?} found in the claims, extracting username from it.");
-        username
-    } else {
-        debug!("Preferred username not found in the claims, extracting from email address.");
-        // Extract the username from the email address
-        let username = email.split('@').next().ok_or(WebError::BadRequest(
-            "Failed to extract username from email address".to_string(),
-        ))?;
-        debug!("Username extracted from email ({email:?}): {username})");
-        username
-    };
-    let settings = Settings::get_current_settings();
-
-    let username = prune_username(username, settings.openid_username_handling);
-    // Check if the username is valid just in case, not everything can be handled by the pruning.
-    check_username(&username)?;
-
     // Get the *sub* claim from the token.
     let sub = token_claims.subject().to_string();
 
@@ -283,7 +302,7 @@ pub(crate) async fn user_from_claims(
                     debug!("User {} tried to log in, but is disabled", user.username);
                     return Err(WebError::Authorization("User is disabled".into()));
                 }
-                // User with the same email already exists, merge the accounts
+                // User with the same email already exists, merge the accounts.
                 info!(
                     "User with email address {} is logging in through OpenID Connect for the \
                     first time and we've found an existing account with the same email \
@@ -294,7 +313,8 @@ pub(crate) async fn user_from_claims(
                 user.save(pool).await?;
                 user
             } else {
-                // Check if the user should be created if they don't exist (default: true)
+                let settings = Settings::get_current_settings();
+                // Check if the user should be created, if doesn't exist (default: true).
                 if !settings.openid_create_account {
                     warn!(
                         "User with email address {} is trying to log in through OpenID Connect \
@@ -309,6 +329,31 @@ pub(crate) async fn user_from_claims(
                     ));
                 }
 
+                // Try to get the username from `preferred_username` claim.
+                // If it's not there, extract it from email.
+                let username = if let Some(username) = token_claims.preferred_username() {
+                    let username = username.as_str();
+                    debug!(
+                        "Preferred username {username} found in the claims. Using the username."
+                    );
+                    username
+                } else {
+                    debug!(
+                        "Preferred username not found in the claims, extracting from email address."
+                    );
+                    // Extract the username from the email address
+                    let username = email.split('@').next().ok_or(WebError::BadRequest(
+                        "Failed to extract username from email address".to_string(),
+                    ))?;
+                    debug!("Username extracted from email ({email:?}): {username})");
+                    username
+                };
+
+                let username = prune_username(username, settings.openid_username_handling);
+                // Check if the username is valid just in case, not everything can be handled by the
+                // pruning.
+                check_username(&username)?;
+
                 info!(
                     "User {username} is logging in through OpenID Connect for the first time and \
                     there is no account with the same email address ({}). Creating a new account.",
@@ -321,11 +366,11 @@ pub(crate) async fn user_from_claims(
                     )));
                 }
 
-                // Extract all necessary information from the token or call the userinfo endpoint
+                // Extract all necessary information from the token or call the userinfo endpoint.
                 let given_name = token_claims
                     .given_name()
-                    // 'None' gets you the default value from a localized claim.
-                    //  Otherwise you would need to pass a locale.
+                    // `None` gets the default value from a localized claim.
+                    // Otherwise, it is required to pass a locale.
                     .and_then(|claim| claim.get(None));
                 let family_name = token_claims.family_name().and_then(|claim| claim.get(None));
                 let phone = token_claims.phone_number();
@@ -341,47 +386,42 @@ pub(crate) async fn user_from_claims(
                     (given_name, family_name, phone)
                 } else {
                     debug!(
-                        "Given name or family name not found in the claims for user {username}, trying to get them \
-                        from the user info endpoint. Current values: given_name: {given_name:?}, family_name: {family_name:?}, phone: {phone:?}"
+                        "Given name or family name not found in the claims for user {username}, \
+                        trying to get them from the user info endpoint. Current values: \
+                        given_name: {given_name:?}, family_name: {family_name:?}, phone: {phone:?}"
                     );
 
-                    let async_http_client = get_async_http_client().await?;
+                    let async_http_client = get_async_http_client()?;
 
-                    let retrieval_error = "Failed to retrieve given name and family name from provider's userinfo endpoint. \
-                        Make sure you have configured your provider correctly and that you have granted the \
-                        necessary permissions to retrieve such information from the token or the userinfo endpoint.";
+                    let retrieval_error = "Failed to retrieve given name and family name from \
+                        provider's userinfo endpoint. Make sure you have configured your provider \
+                        correctly and that you have granted the necessary permissions to retrieve \
+                        such information from the token or the userinfo endpoint.";
                     userinfo_response = core_client
                         .user_info(access_token.clone(), Some(token_claims.subject().clone()))
-                        .map_err(
-                            |err| {
-                                error!(
-                                    "Failed to get family name and given name from provider's userinfo endpoint, they may not support this. Error details: {err:?}",
-                                );
-
-                                WebError::BadRequest(
-                                    retrieval_error.into(),
-                                )
-                            }
-                        )?
+                        .map_err(|err| {
+                            error!(
+                                "Failed to get family name and given name from provider's \
+                                userinfo endpoint, they may not support this. Error details: {err}"
+                            );
+                            WebError::BadRequest(retrieval_error.into())
+                        })?
                         .request_async(&async_http_client)
                         .await
-                        .map_err(
-                            |err| {
-                                error!(
-                                    "Failed to get family name and given name from provider's userinfo endpoint. Error details: {err:?}",
-                                );
-
-                                WebError::BadRequest(
-                                    retrieval_error.into(),
-                                )
-                            }
-                        )?;
+                        .map_err(|err| {
+                            error!(
+                                "Failed to get family name and given name from provider's userinfo \
+                                endpoint. Error details: {err}",
+                            );
+                            WebError::BadRequest(retrieval_error.into())
+                        })?;
 
                     let claim_error = |claim_name: &str| {
                         format!(
-                            "Failed to retrieve {claim_name} from provider's userinfo endpoint and the ID token. \
-                            Make sure you have configured your provider correctly and that you have \
-                            granted the necessary permissions to retrieve such information from the token or the userinfo endpoint.",
+                            "Failed to retrieve {claim_name} from provider's userinfo endpoint and \
+                            the ID token.  Make sure you have configured your provider correctly \
+                            and that you have granted the necessary permissions to retrieve such \
+                            information from the token or the userinfo endpoint.",
                         )
                     };
                     let given_name = userinfo_response
@@ -395,7 +435,8 @@ pub(crate) async fn user_from_claims(
                     let phone = userinfo_response.phone_number();
 
                     debug!(
-                        "Given name and family name successfully retrieved from the user info endpoint for user {username}."
+                        "Given name and family name successfully retrieved from the user info \
+                        endpoint for user {username}."
                     );
 
                     (given_name, family_name, phone)
@@ -435,15 +476,24 @@ pub(crate) async fn get_auth_info(
     let (_client_id, client) = make_oidc_client(config.callback_url(), &provider).await?;
 
     // Generate the redirect URL and the values needed later for callback authenticity verification
-    let (authorize_url, csrf_state, nonce) = client
+    let mut authorize_url_builder = client
         .authorize_url(
             CoreAuthenticationFlow::AuthorizationCode,
             CsrfToken::new_random,
             Nonce::new_random,
         )
         .add_scope(Scope::new("email".into()))
-        .add_scope(Scope::new("profile".into()))
-        .url();
+        .add_scope(Scope::new("profile".into()));
+
+    if SELECT_ACCOUNT_SUPPORTED_PROVIDERS
+        .iter()
+        .any(|&p| provider.name.eq_ignore_ascii_case(p))
+    {
+        authorize_url_builder =
+            authorize_url_builder.add_prompt(openidconnect::core::CoreAuthPrompt::SelectAccount);
+    }
+
+    let (authorize_url, csrf_state, nonce) = authorize_url_builder.url();
 
     let cookie_domain = config
         .cookie_domain
@@ -471,10 +521,7 @@ pub(crate) async fn get_auth_info(
         private_cookies,
         ApiResponse {
             json: json!(
-                {
-                    "url": authorize_url,
-                    "button_display_name": provider.display_name
-                }
+                {"url": authorize_url, "button_display_name": provider.display_name}
             ),
             status: StatusCode::OK,
         },
@@ -551,14 +598,16 @@ pub(crate) async fn auth_callback(
         .max_age(max_age);
     let cookies = cookies.add(auth_cookie);
 
-    // The user may not be yet authorized (pre-MFA) but syncing their groups should be fine here, since he already managed to login through the provider.
-    // There is currently no other way to sync the groups for the MFA enabled user logging in through the provider without firing it
-    // on every login attempt, even for standard, non-provider users.
+    // The user may not yet be authorized (pre-MFA), but syncing their groups should be fine here,
+    // since he already managed to login through the provider. Currently, there is no other way to
+    // sync the groups for the MFA enabled user logging in through the provider without firing it on
+    // every login attempt, even for standard, non-provider users.
     if let Err(err) =
         sync_user_groups_if_configured(&user, &appstate.pool, &appstate.wireguard_tx).await
     {
         error!(
-            "Failed to sync user groups for user {} with the directory while the user was trying to login in through an external provider: {err:?}",
+            "Failed to sync user groups for user {} with the directory while the user was trying \
+            to login in through an external provider: {err}",
             user.username
         );
     } else {
@@ -673,4 +722,54 @@ mod test {
             "averylongnameeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee"
         );
     }
+
+    #[test]
+    fn test_state_build_and_extract() {
+        // without data
+        let token = build_state(None);
+        let decoded = BASE64_STANDARD.decode(token.secret());
+        // not base64 encoded
+        assert!(decoded.is_err());
+        assert!(!token.secret().is_empty());
+
+        // with data
+        let data = "somedata".to_string();
+        let token = build_state(Some(data.clone()));
+        let decoded = BASE64_STANDARD.decode(token.secret());
+        assert!(decoded.is_ok());
+        let decoded_str = String::from_utf8(decoded.unwrap()).unwrap();
+        let (csrf, state_data) = decoded_str.split_once('.').unwrap();
+        assert!(!csrf.is_empty());
+        assert_eq!(state_data, data);
+
+        // valid
+        let data = "my_state_data".to_string();
+        let token = build_state(Some(data.clone()));
+        let extracted = extract_state_data(token.secret());
+        assert_eq!(extracted, Some(data));
+
+        // invalid base64
+        let extracted = extract_state_data("not_base64!!");
+        assert_eq!(extracted, None);
+
+        // no dot
+        let encoded = BASE64_STANDARD.encode("no_dot_here");
+        let extracted = extract_state_data(&encoded);
+        assert_eq!(extracted, None);
+
+        // empty first part
+        let encoded = BASE64_STANDARD.encode(".somedata");
+        let extracted = extract_state_data(&encoded);
+        assert_eq!(extracted, None);
+
+        // empty second part
+        let encoded = BASE64_STANDARD.encode("csrf.");
+        let extracted = extract_state_data(&encoded);
+        assert_eq!(extracted, Some("".to_string()));
+
+        // multiple dots
+        let encoded = BASE64_STANDARD.encode("csrf.data.with.dots");
+        let extracted = extract_state_data(&encoded);
+        assert_eq!(extracted, Some("data.with.dots".to_string()));
+    }
 }
