@@ -54,11 +54,18 @@
 //!
 use std::collections::{HashMap, HashSet};
 
-use sqlx::{PgConnection, PgPool, Type};
+use defguard_common::db::{
+    Id,
+    models::{
+        Settings,
+        settings::{LdapSyncStatus, update_current_settings},
+    },
+};
+use sqlx::{PgConnection, PgPool};
 
-use super::{error::LdapError, LDAPConfig};
+use super::{LDAPConfig, error::LdapError};
 use crate::{
-    db::{models::settings::update_current_settings, Group, Id, Settings, User},
+    db::{Group, User},
     hashset,
 };
 
@@ -85,26 +92,13 @@ pub enum Authority {
     Defguard,
 }
 
-#[derive(Clone, Debug, Copy, Eq, PartialEq, Deserialize, Serialize, Default, Type)]
-#[sqlx(type_name = "ldap_sync_status", rename_all = "lowercase")]
-pub enum SyncStatus {
-    InSync,
-    #[default]
-    OutOfSync,
-}
-
-impl SyncStatus {
-    pub fn is_out_of_sync(&self) -> bool {
-        matches!(self, SyncStatus::OutOfSync)
-    }
-}
-
-pub fn get_ldap_sync_status() -> SyncStatus {
+#[must_use]
+pub fn get_ldap_sync_status() -> LdapSyncStatus {
     let settings = Settings::get_current_settings();
     settings.ldap_sync_status
 }
 
-pub async fn set_ldap_sync_status(status: SyncStatus, pool: &PgPool) -> Result<(), LdapError> {
+pub async fn set_ldap_sync_status(status: LdapSyncStatus, pool: &PgPool) -> Result<(), LdapError> {
     debug!("Setting LDAP sync status to {status:?}");
     let mut settings = Settings::get_current_settings();
     settings.ldap_sync_status = status;
@@ -113,6 +107,7 @@ pub async fn set_ldap_sync_status(status: SyncStatus, pool: &PgPool) -> Result<(
     Ok(())
 }
 
+#[must_use]
 pub fn is_ldap_desynced() -> bool {
     get_ldap_sync_status().is_out_of_sync()
 }
@@ -244,7 +239,7 @@ pub(super) fn compute_group_sync_changes<'a>(
                         ldap_config.user_dn_from_user(m) == ldap_config.user_dn_from_user(u)
                     })
                 })
-                .cloned()
+                .copied()
                 .collect::<HashSet<_>>();
 
             let missing_from_ldap = members
@@ -261,43 +256,55 @@ pub(super) fn compute_group_sync_changes<'a>(
                 "Group {group:?} members missing from Defguard: {missing_from_defguard:?}, missing from LDAP: {missing_from_ldap:?}"
             );
 
-            if !missing_from_defguard.is_empty() {
+            if missing_from_defguard.is_empty() {
+                debug!("Group {group:?} has no members missing from Defguard");
+            } else {
                 match authority {
                     Authority::Defguard => {
-                        debug!("Group {group:?} has members missing from Defguard, marking them for deletion in LDAP: {missing_from_defguard:?}");
+                        debug!(
+                            "Group {group:?} has members missing from Defguard, marking them for deletion in LDAP: {missing_from_defguard:?}"
+                        );
                         delete_ldap.insert(group.clone(), missing_from_defguard);
                     }
                     Authority::LDAP => {
-                        debug!("Group {group:?} has members missing from Defguard, marking them for addition in Defguard: {missing_from_defguard:?}");
+                        debug!(
+                            "Group {group:?} has members missing from Defguard, marking them for addition in Defguard: {missing_from_defguard:?}"
+                        );
                         add_defguard.insert(group.clone(), missing_from_defguard);
                     }
                 }
-            } else {
-                debug!("Group {group:?} has no members missing from Defguard");
             }
 
-            if !missing_from_ldap.is_empty() {
+            if missing_from_ldap.is_empty() {
+                debug!("Group {group:?} has no members missing from LDAP");
+            } else {
                 match authority {
                     Authority::Defguard => {
-                        debug!("Group {group:?} has members missing from LDAP, marking them for addition to LDAP: {missing_from_ldap:?}");
+                        debug!(
+                            "Group {group:?} has members missing from LDAP, marking them for addition to LDAP: {missing_from_ldap:?}"
+                        );
                         add_ldap.insert(group.clone(), missing_from_ldap);
                     }
                     Authority::LDAP => {
-                        debug!("Group {group:?} has members missing from LDAP, marking them for deletion in Defguard: {missing_from_ldap:?}");
+                        debug!(
+                            "Group {group:?} has members missing from LDAP, marking them for deletion in Defguard: {missing_from_ldap:?}"
+                        );
                         delete_defguard.insert(group.clone(), missing_from_ldap);
                     }
                 }
-            } else {
-                debug!("Group {group:?} has no members missing from LDAP");
             }
         } else {
             match authority {
                 Authority::Defguard => {
-                    debug!("Group {group:?} is missing from LDAP, marking it for addition to LDAP along with all members due to Defguard authority");
+                    debug!(
+                        "Group {group:?} is missing from LDAP, marking it for addition to LDAP along with all members due to Defguard authority"
+                    );
                     add_ldap.insert(group.clone(), members);
                 }
                 Authority::LDAP => {
-                    debug!("Group {group:?} is missing from LDAP, marking all its member for deletion from Defguard due to LDAP authority");
+                    debug!(
+                        "Group {group:?} is missing from LDAP, marking all its member for deletion from Defguard due to LDAP authority"
+                    );
                     delete_defguard.insert(group.clone(), members);
                 }
             }
@@ -308,11 +315,15 @@ pub(super) fn compute_group_sync_changes<'a>(
         if !defguard_memberships.contains_key(&group) {
             match authority {
                 Authority::Defguard => {
-                    debug!("Group {group:?} is missing from Defguard, marking all its member for deletion from LDAP due to Defguard authority");
+                    debug!(
+                        "Group {group:?} is missing from Defguard, marking all its member for deletion from LDAP due to Defguard authority"
+                    );
                     delete_ldap.insert(group, members);
                 }
                 Authority::LDAP => {
-                    debug!("Group {group:?} is missing from Defguard, marking all its member for addition to Defguard due to LDAP authority");
+                    debug!(
+                        "Group {group:?} is missing from Defguard, marking all its member for addition to Defguard due to LDAP authority"
+                    );
                     add_defguard.insert(group, members);
                 }
             }
@@ -402,7 +413,7 @@ pub(super) fn extract_intersecting_users(
         }
     }
 
-    for user in intersecting_users_ldap.into_iter() {
+    for user in intersecting_users_ldap {
         if let Some(defguard_user) = defguard_users
             .iter()
             .position(|u| ldap_config.user_dn_from_user(u) == ldap_config.user_dn_from_user(&user))
@@ -417,6 +428,7 @@ pub(super) fn extract_intersecting_users(
 
 const DEFAULT_LDAP_SYNC_INTERVAL: u64 = 60 * 5;
 
+#[must_use]
 pub fn get_ldap_sync_interval() -> u64 {
     let settings = Settings::get_current_settings();
     settings
@@ -435,7 +447,7 @@ impl super::LDAPConnection {
     ) -> Result<(), LdapError> {
         let mut transaction = pool.begin().await?;
 
-        for (ldap_user, defguard_user) in intersecting_users.iter_mut() {
+        for (ldap_user, defguard_user) in &mut intersecting_users {
             if attrs_different(defguard_user, ldap_user, &self.config) {
                 debug!(
                     "User {defguard_user} attributes differ between LDAP and Defguard, merging..."
@@ -622,7 +634,10 @@ impl super::LDAPConnection {
             }
         }
 
-        debug!("The following groups were defined for sync: {:?}, only Defguard users belonging to these groups will be synced", sync_groups);
+        debug!(
+            "The following groups were defined for sync: {:?}, only Defguard users belonging to these groups will be synced",
+            sync_groups
+        );
         let mut sync_group_members = HashSet::new();
         for sync_group in &sync_groups {
             let members = sync_group.members(pool).await?;
@@ -841,7 +856,7 @@ impl super::LDAPConnection {
             self.delete_user(&user).await?;
         }
 
-        for user in changes.add_ldap.iter_mut() {
+        for user in &mut changes.add_ldap {
             debug!("Adding user {} to LDAP", user.username);
             self.add_user(user, None, pool).await?;
         }
@@ -868,10 +883,9 @@ impl super::LDAPConnection {
                 Ok(user) => all_users.push(user),
                 Err(err) => {
                     warn!(
-                        "Failed to create user {} from LDAP entry, error: {}. The user will be skipped during sync",
-                        username, err
+                        "Failed to create user {username} from LDAP entry, error: {err}. The user will be skipped during sync"
                     );
-                    debug!("Skipping user {} due to error: {}", username, err);
+                    debug!("Skipping user {username} due to error: {err}");
                 }
             }
         }
