@@ -1,41 +1,56 @@
 use std::collections::HashSet;
 
-use sqlx::{PgPool, Transaction};
+use defguard_common::{
+    csv::AsCsv,
+    db::{
+        Id,
+        models::{BiometricAuth, MFAMethod, Settings, settings::defaults::WELCOME_EMAIL_SUBJECT},
+    },
+};
+use defguard_mail::{
+    Mail,
+    templates::{self, TemplateLocation},
+};
+use sqlx::{PgPool, Transaction, query_scalar};
 use tokio::sync::{
     broadcast::Sender,
-    mpsc::{error::SendError, UnboundedSender},
+    mpsc::{UnboundedSender, error::SendError},
 };
 use tonic::Status;
 
-use super::{
-    proto::proxy::{
-        ActivateUserRequest, AdminInfo, Device as ProtoDevice, DeviceConfig as ProtoDeviceConfig,
-        DeviceConfigResponse, EnrollmentStartRequest, EnrollmentStartResponse, ExistingDevice,
-        InitialUserInfo, NewDevice,
-    },
-    InstanceInfo,
-};
+use super::InstanceInfo;
 use crate::{
     db::{
+        Device, GatewayEvent, User, WireguardNetwork,
         models::{
             device::{DeviceConfig, DeviceInfo, DeviceType},
-            enrollment::{Token, TokenError, ENROLLMENT_TOKEN_TYPE},
+            enrollment::{ENROLLMENT_TOKEN_TYPE, Token, TokenError},
             polling_token::PollingToken,
+            wireguard::LocationMfaMode,
         },
-        Device, GatewayEvent, Id, Settings, User, WireguardNetwork,
     },
     enterprise::{
-        db::models::enterprise_settings::EnterpriseSettings, ldap::utils::ldap_add_user,
+        db::models::{enterprise_settings::EnterpriseSettings, openid_provider::OpenIdProvider},
+        ldap::utils::ldap_add_user,
         limits::update_counts,
     },
     events::{BidiRequestContext, BidiStreamEvent, BidiStreamEventType, EnrollmentEvent},
     grpc::utils::{build_device_config_response, new_polling_token, parse_client_info},
-    handlers::{mail::send_new_device_added_email, user::check_password_strength},
+    handlers::{
+        mail::{
+            send_email_mfa_activation_email, send_mfa_configured_email, send_new_device_added_email,
+        },
+        user::check_password_strength,
+    },
     headers::get_device_info,
-    mail::Mail,
-    server_config,
-    templates::{self, TemplateLocation},
-    AsCsv,
+    is_valid_phone_number, server_config,
+};
+use defguard_proto::proxy::{
+    ActivateUserRequest, AdminInfo, CodeMfaSetupFinishRequest, CodeMfaSetupFinishResponse,
+    CodeMfaSetupStartRequest, CodeMfaSetupStartResponse, Device as ProtoDevice,
+    DeviceConfig as ProtoDeviceConfig, DeviceConfigResponse, EnrollmentStartRequest,
+    EnrollmentStartResponse, ExistingDevice, InitialUserInfo,
+    LocationMfaMode as ProtoLocationMfaMode, MfaMethod, NewDevice, RegisterMobileAuthRequest,
 };
 
 pub(super) struct EnrollmentServer {
@@ -105,7 +120,7 @@ impl EnrollmentServer {
     ) -> Result<(), SendError<BidiStreamEvent>> {
         let event = BidiStreamEvent {
             context,
-            event: BidiStreamEventType::Enrollment(event),
+            event: BidiStreamEventType::Enrollment(Box::new(event)),
         };
 
         self.bidi_event_tx.send(event)
@@ -115,7 +130,7 @@ impl EnrollmentServer {
     pub async fn start_enrollment(
         &self,
         request: EnrollmentStartRequest,
-        info: Option<super::proto::proxy::DeviceInfo>,
+        info: Option<defguard_proto::proxy::DeviceInfo>,
     ) -> Result<EnrollmentStartResponse, Status> {
         debug!("Starting enrollment session, request: {request:?}");
         // fetch enrollment token
@@ -194,7 +209,20 @@ impl EnrollmentServer {
                 "Retrieving instance info for user {}({:?}).",
                 user.username, user.id
             );
-            let instance_info = InstanceInfo::new(settings, &user.username, &enterprise_settings);
+
+            let openid_provider = OpenIdProvider::get_current(&self.pool)
+                .await
+                .map_err(|err| {
+                    error!("Failed to get OpenID provider: {err}");
+                    Status::internal(format!("unexpected error: {err}"))
+                })?;
+            let smtp_configured = settings.smtp_configured();
+            let instance_info = InstanceInfo::new(
+                settings,
+                &user.username,
+                &enterprise_settings,
+                openid_provider,
+            );
             debug!("Instance info {instance_info:?}");
 
             debug!(
@@ -202,7 +230,7 @@ impl EnrollmentServer {
                 user.username, user.id
             );
             let (username, user_id) = (user.username.clone(), user.id);
-            let user_info = InitialUserInfo::from_user(&self.pool, user)
+            let user_info = initial_info_from_user(&self.pool, user)
                 .await
                 .map_err(|err| {
                     error!(
@@ -217,10 +245,7 @@ impl EnrollmentServer {
             let admin_info = admin.map(AdminInfo::from);
             debug!("Admin info {admin_info:?}");
 
-            debug!(
-                "Creating enrollment start response for user {}({:?}).",
-                username, user_id,
-            );
+            debug!("Creating enrollment start response for user {username}({user_id:?}).");
             let enterprise_settings =
                 EnterpriseSettings::get(&mut *transaction)
                     .await
@@ -228,11 +253,24 @@ impl EnrollmentServer {
                         error!("Failed to get enterprise settings: {err}");
                         Status::internal("unexpected error")
                     })?;
-            let enrollment_settings = super::proto::proxy::Settings {
+            // check if any locations enforce internal MFA
+            let instance_has_internal_mfa = query_scalar!(
+                "SELECT EXISTS( \
+                    SELECT 1 FROM wireguard_network \
+                    WHERE location_mfa_mode = 'internal'::location_mfa_mode \
+                ) \"exists!\""
+            )
+            .fetch_one(&self.pool)
+            .await
+            .map_err(|_| Status::internal("Failed to read data".to_string()))?;
+            let enrollment_settings = defguard_proto::proxy::EnrollmentSettings {
                 vpn_setup_optional,
+                smtp_configured,
                 only_client_activation: enterprise_settings.only_client_activation,
+                admin_device_management: enterprise_settings.admin_device_management,
+                mfa_required: instance_has_internal_mfa,
             };
-            let response = super::proto::proxy::EnrollmentStartResponse {
+            let response = defguard_proto::proxy::EnrollmentStartResponse {
                 admin: admin_info,
                 user: Some(user_info),
                 deadline_timestamp: session_deadline.and_utc().timestamp(),
@@ -265,14 +303,68 @@ impl EnrollmentServer {
         }
     }
 
+    #[instrument(skip_all)]
+    pub async fn register_mobile_auth(
+        &self,
+        request: RegisterMobileAuthRequest,
+    ) -> Result<(), Status> {
+        debug!("Register mobile auth started");
+        let enrollment = self.validate_session(Some(&request.token)).await?;
+        let user = enrollment.fetch_user(&self.pool).await?;
+        Device::validate_pubkey(&request.device_pub_key).map_err(|err| {
+            error!(
+                "Invalid public key {}, device won't be registered as mobile MFA auth for user {}\
+                ({:?}): {err}",
+                request.device_pub_key, user.username, user.id
+            );
+            Status::invalid_argument("invalid pubkey")
+        })?;
+        let Some(device) = Device::find_by_pubkey(&self.pool, &request.device_pub_key)
+            .await
+            .map_err(|err| {
+                error!("Failed to read devices from db: {err}");
+                Status::internal("Something went wrong")
+            })?
+        else {
+            return Err(Status::invalid_argument(
+                "Device with given public key doesn't exist",
+            ));
+        };
+        BiometricAuth::validate_pubkey(&request.device_pub_key)?;
+        let mobile_auth = BiometricAuth::new(device.id, request.auth_pub_key);
+        let _ = mobile_auth.save(&self.pool).await.map_err(|err| {
+            error!("Failed to save mobile auth into db: {err}");
+            Status::internal("Failed to save results")
+        })?;
+        info!(
+            "User {}({}) registered mobile auth for device {}({})",
+            user.username, user.id, device.name, device.id
+        );
+        Ok(())
+    }
+
+    fn validate_activated_user(request: &ActivateUserRequest) -> Result<(), Status> {
+        if let Some(ref phone_number) = request.phone_number {
+            if !is_valid_phone_number(phone_number) {
+                return Err(Status::new(
+                    tonic::Code::InvalidArgument,
+                    "invalid phone number",
+                ));
+            }
+        }
+
+        Ok(())
+    }
+
     #[instrument(skip_all)]
     pub async fn activate_user(
         &self,
         request: ActivateUserRequest,
-        req_device_info: Option<super::proto::proxy::DeviceInfo>,
+        req_device_info: Option<defguard_proto::proxy::DeviceInfo>,
     ) -> Result<(), Status> {
-        debug!("Activating user account: {request:?}");
+        debug!("Activating user account");
         let enrollment = self.validate_session(request.token.as_ref()).await?;
+        Self::validate_activated_user(&request)?;
 
         let ip_address;
         let device_info;
@@ -334,7 +426,7 @@ impl EnrollmentServer {
 
         debug!("Retriving settings to send welcome email...");
         let settings = Settings::get_current_settings();
-        debug!("Successfully retrived settings.");
+        debug!("Settings successfully retrieved.");
 
         // send welcome email
         debug!("Try to send welcome email...");
@@ -391,14 +483,42 @@ impl EnrollmentServer {
     pub async fn create_device(
         &self,
         request: NewDevice,
-        req_device_info: Option<super::proto::proxy::DeviceInfo>,
+        req_device_info: Option<defguard_proto::proxy::DeviceInfo>,
     ) -> Result<DeviceConfigResponse, Status> {
-        debug!("Adding new user device: {request:?}");
+        debug!("Adding new user device");
         let enrollment_token = self.validate_session(request.token.as_ref()).await?;
 
         // fetch related users
         let user = enrollment_token.fetch_user(&self.pool).await?;
 
+        // check if adding device by non-admin users is allowed
+        debug!(
+            "Fetching enterprise settings for device creation process for user {}({:?})",
+            user.username, user.id,
+        );
+        let enterprise_settings = EnterpriseSettings::get(&self.pool).await.map_err(|err| {
+            error!(
+            "Failed to fetch enterprise settings for device creation process for user {}({:?}): \
+            {err}",
+            user.username, user.id,
+        );
+            Status::internal("unexpected error")
+        })?;
+        debug!("Enterprise settings: {enterprise_settings:?}");
+
+        if !user.is_admin(&self.pool).await.map_err(|err| {
+            error!(
+                "Failed to fetch admin status for user {}({:?}): {err}",
+                user.username, user.id,
+            );
+            Status::internal("unexpected error")
+        })? && enterprise_settings.admin_device_management
+        {
+            return Err(Status::invalid_argument(
+                "only admin users can manage devices",
+            ));
+        }
+
         // add device
         debug!(
             "Verifying if user {}({:?}) is active",
@@ -463,11 +583,7 @@ impl EnrollmentServer {
         {
             warn!(
                 "User {}({:?}) failed to add device {}, identical pubkey ({}) already exists for device {}",
-                user.username,
-                user.id,
-                request.name,
-                request.pubkey,
-                device.name
+                user.username, user.id, request.name, request.pubkey, device.name
             );
             return Err(Status::invalid_argument("invalid key"));
         }
@@ -632,7 +748,10 @@ impl EnrollmentServer {
                         Status::internal("unexpected error")
                     })?
                 {
-                    debug!("Sending firewall config update for location {location} affected by adding new device {}, user {}({})", device.wireguard_pubkey, user.username, user.id);
+                    debug!(
+                        "Sending firewall config update for location {location} affected by adding new device {}, user {}({})",
+                        device.wireguard_pubkey, user.username, user.id
+                    );
                     self.send_wireguard_event(GatewayEvent::FirewallConfigChanged(
                         location_id,
                         firewall_config,
@@ -661,23 +780,6 @@ impl EnrollmentServer {
         let settings = Settings::get_current_settings();
         debug!("Settings: {settings:?}");
 
-        debug!(
-            "Fetching enterprise settings for device {} creation process for user {}({:?})",
-            device.wireguard_pubkey, user.username, user.id,
-        );
-        let enterprise_settings =
-            EnterpriseSettings::get(&mut *transaction)
-                .await
-                .map_err(|err| {
-                    error!(
-            "Failed to fetch enterprise settings for device {} creation process for user {}({:?}): \
-            {err}",
-            device.wireguard_pubkey, user.username, user.id,
-        );
-                    Status::internal("unexpected error")
-                })?;
-        debug!("Enterprise settings: {enterprise_settings:?}");
-
         // create polling token for further client communication
         debug!(
             "Creating polling token for further client communication for device {}, user {}({:?})",
@@ -731,15 +833,27 @@ impl EnrollmentServer {
 
         info!("Device {} remote configuration done.", device.name);
 
+        let openid_provider = OpenIdProvider::get_current(&self.pool)
+            .await
+            .map_err(|err| {
+                error!("Failed to get OpenID provider: {err}");
+                Status::internal(format!("unexpected error: {err}"))
+            })?;
+
         let response = DeviceConfigResponse {
             device: Some(device.clone().into()),
             configs: configs.into_iter().map(Into::into).collect(),
             instance: Some(
-                InstanceInfo::new(settings, &user.username, &enterprise_settings).into(),
+                InstanceInfo::new(
+                    settings,
+                    &user.username,
+                    &enterprise_settings,
+                    openid_provider,
+                )
+                .into(),
             ),
             token: Some(token.token),
         };
-        debug!("{response:?}.");
 
         // Prepare event context and push the event
         let (ip, user_agent) = parse_client_info(&req_device_info).map_err(Status::internal)?;
@@ -760,7 +874,7 @@ impl EnrollmentServer {
         request: ExistingDevice,
     ) -> Result<DeviceConfigResponse, Status> {
         debug!("Getting network info for device: {:?}", request.pubkey);
-        let _token = self.validate_session(request.token.as_ref()).await?;
+        let token = self.validate_session(request.token.as_ref()).await?;
 
         Device::validate_pubkey(&request.pubkey).map_err(|_| {
             error!("Invalid pubkey {}", &request.pubkey);
@@ -769,12 +883,148 @@ impl EnrollmentServer {
         // Find existing device by public key.
         let Ok(Some(device)) = Device::find_by_pubkey(&self.pool, &request.pubkey).await else {
             error!("Failed to fetch device by pubkey: {}", &request.pubkey);
-            return Err(Status::internal("device not found"));
+            return Err(Status::not_found("device not found"));
         };
 
+        // check if device owner matches used enrollment token
+        if device.user_id != token.user_id {
+            error!(
+                "Enrollment token does not match device with pubkey {}",
+                request.pubkey
+            );
+            return Err(Status::unauthenticated(
+                "enrollment token is not valid for specified device",
+            ));
+        }
+
         let token = new_polling_token(&self.pool, &device).await?;
         build_device_config_response(&self.pool, device, Some(token)).await
     }
+
+    // TODO: Add events
+    #[instrument(skip_all)]
+    pub(crate) async fn register_code_mfa_start(
+        &self,
+        request: CodeMfaSetupStartRequest,
+    ) -> Result<CodeMfaSetupStartResponse, Status> {
+        debug!("Begin enrollment code mfa setup start");
+        let method = request.method();
+        if method != MfaMethod::Email && method != MfaMethod::Totp {
+            return Err(Status::invalid_argument("Method not supported".to_string()));
+        }
+        let enrollment = Token::find_by_id(&self.pool, &request.token).await?;
+        let mut user = enrollment.fetch_user(&self.pool).await?;
+        // available only for unenrolled users
+        if user.is_enrolled() {
+            return Err(Status::permission_denied("User is already enrolled"));
+        }
+        match method {
+            MfaMethod::Email => {
+                let settings = Settings::get_current_settings();
+                if !settings.smtp_configured() {
+                    error!("Unable to start Email mfa setup. SMTP is not configured");
+                    return Err(Status::internal("SMTP not configured".to_string()));
+                }
+                if user.email_mfa_enabled {
+                    return Err(Status::invalid_argument(
+                        "Method already enabled".to_string(),
+                    ));
+                }
+                user.new_email_secret(&self.pool).await.map_err(|_| {
+                    error!("Failed to create email secret");
+                    Status::internal("Failed to setup email mfa".to_string())
+                })?;
+                info!("Created email secret for {}", &user.username);
+                send_email_mfa_activation_email(&user, &self.mail_tx, None).map_err(|e| {
+                    error!("Failed to send email mfa activation email.\nReason:{e}");
+                    Status::internal("Failed to send activation email".to_string())
+                })?;
+                Ok(CodeMfaSetupStartResponse { totp_secret: None })
+            }
+            MfaMethod::Totp => {
+                if user.totp_enabled {
+                    return Err(Status::invalid_argument(
+                        "Method already enabled".to_string(),
+                    ));
+                }
+                let secret = user.new_totp_secret(&self.pool).await.map_err(|_| {
+                    error!("Failed to make new totp secret");
+                    Status::internal(String::new())
+                })?;
+                info!("New totp secret created for {}", &user.username);
+                Ok(CodeMfaSetupStartResponse {
+                    totp_secret: Some(secret),
+                })
+            }
+            _ => Err(Status::invalid_argument("Method not supported".to_string())),
+        }
+    }
+
+    // TODO: Add events
+    #[instrument(skip_all)]
+    pub(crate) async fn register_code_mfa_finish(
+        &self,
+        request: CodeMfaSetupFinishRequest,
+    ) -> Result<CodeMfaSetupFinishResponse, Status> {
+        debug!("Begin enrollment code mfa setup finish");
+        let enrollment = self.validate_session(Some(&request.token)).await?;
+        let method = request.method();
+        if method != MfaMethod::Totp && method != MfaMethod::Email {
+            return Err(Status::invalid_argument("Method not supported"));
+        }
+        let mut user = enrollment.fetch_user(&self.pool).await?;
+        if user.mfa_enabled {
+            return Err(Status::invalid_argument(
+                "Mfa already enabled on the account".to_string(),
+            ));
+        }
+        // available only for unenrolled users
+        if user.is_enrolled() {
+            return Err(Status::permission_denied("User is already enrolled"));
+        }
+        let mfa_method: MFAMethod;
+        // enable corresponding MFA
+        match method {
+            MfaMethod::Email => {
+                if !user.verify_email_mfa_code(&request.code) {
+                    return Err(Status::invalid_argument("Email code invalid".to_string()));
+                }
+                user.enable_email_mfa(&self.pool)
+                    .await
+                    .map_err(|_| Status::internal("Enabling method failed.".to_string()))?;
+                mfa_method = MFAMethod::Email;
+            }
+            MfaMethod::Totp => {
+                if !user.verify_totp_code(&request.code) {
+                    return Err(Status::invalid_argument("Code invalid".to_string()));
+                }
+                user.enable_totp(&self.pool)
+                    .await
+                    .map_err(|_| Status::internal("Enabling method failed.".to_string()))?;
+                mfa_method = MFAMethod::OneTimePassword;
+            }
+            _ => {
+                return Err(Status::invalid_argument("Method not supported"));
+            }
+        }
+        user.enable_mfa(&self.pool)
+            .await
+            .map_err(|_| Status::internal("Enabling MFA on the account failed.".to_string()))?;
+        let recovery_codes = user
+            .get_recovery_codes(&self.pool)
+            .await
+            .map_err(|_| Status::internal("Failed to get recovery codes.".to_string()))?
+            .ok_or_else(|| Status::internal("Recovery codes not found".to_string()))?;
+        if let Err(e) = send_mfa_configured_email(None, &user, &mfa_method, &self.mail_tx) {
+            error!("Failed to send mfa configured email\nReason: {e}");
+        }
+        info!(
+            "Successfully enabled MFA method {} for user {}",
+            method.as_str_name(),
+            &user.username
+        );
+        Ok(CodeMfaSetupFinishResponse { recovery_codes })
+    }
 }
 
 impl From<User<Id>> for AdminInfo {
@@ -787,26 +1037,31 @@ impl From<User<Id>> for AdminInfo {
     }
 }
 
-impl InitialUserInfo {
-    async fn from_user(pool: &PgPool, user: User<Id>) -> Result<Self, sqlx::Error> {
-        let enrolled = user.is_enrolled();
-        let devices = user.user_devices(pool).await?;
-        let device_names = devices.into_iter().map(|dev| dev.device.name).collect();
-        Ok(Self {
-            first_name: user.first_name,
-            last_name: user.last_name,
-            login: user.username,
-            email: user.email,
-            phone_number: user.phone,
-            is_active: user.is_active,
-            device_names,
-            enrolled,
-        })
-    }
+async fn initial_info_from_user(
+    pool: &PgPool,
+    user: User<Id>,
+) -> Result<InitialUserInfo, sqlx::Error> {
+    let enrolled = user.is_enrolled();
+    let devices = user.user_devices(pool).await?;
+    let device_names = devices.into_iter().map(|dev| dev.device.name).collect();
+    let is_admin = user.is_admin(pool).await?;
+    Ok(InitialUserInfo {
+        first_name: user.first_name,
+        last_name: user.last_name,
+        login: user.username,
+        email: user.email,
+        phone_number: user.phone,
+        is_active: user.is_active,
+        device_names,
+        enrolled,
+        is_admin,
+    })
 }
 
 impl From<DeviceConfig> for ProtoDeviceConfig {
     fn from(config: DeviceConfig) -> Self {
+        // DEPRECATED(1.5): superseeded by location_mfa_mode
+        let mfa_enabled = config.location_mfa_mode == LocationMfaMode::Internal;
         Self {
             network_id: config.network_id,
             network_name: config.network_name,
@@ -816,8 +1071,13 @@ impl From<DeviceConfig> for ProtoDeviceConfig {
             pubkey: config.pubkey,
             allowed_ips: config.allowed_ips.as_csv(),
             dns: config.dns,
-            mfa_enabled: config.mfa_enabled,
             keepalive_interval: config.keepalive_interval,
+            #[allow(deprecated)]
+            mfa_enabled,
+            location_mfa_mode: Some(
+                <LocationMfaMode as Into<ProtoLocationMfaMode>>::into(config.location_mfa_mode)
+                    .into(),
+            ),
         }
     }
 }
@@ -848,7 +1108,10 @@ impl Token {
         debug!("Sending welcome mail to {}", user.username);
         let mail = Mail {
             to: user.email.clone(),
-            subject: settings.enrollment_welcome_email_subject.clone().unwrap(),
+            subject: settings
+                .enrollment_welcome_email_subject
+                .clone()
+                .unwrap_or_else(|| WELCOME_EMAIL_SUBJECT.to_string()),
             content: self
                 .get_welcome_email_content(&mut *transaction, ip_address, device_info)
                 .await?,
@@ -883,8 +1146,8 @@ impl Token {
             to: admin.email.clone(),
             subject: "[defguard] User enrollment completed".into(),
             content: templates::enrollment_admin_notification(
-                user,
-                admin,
+                &user.clone().into(),
+                &admin.clone().into(),
                 ip_address,
                 device_info,
             )?,
@@ -906,3 +1169,121 @@ impl Token {
         }
     }
 }
+
+#[cfg(test)]
+mod test {
+    use defguard_common::{
+        config::{DefGuardConfig, SERVER_CONFIG},
+        db::{
+            models::{
+                Settings,
+                settings::{defaults::WELCOME_EMAIL_SUBJECT, initialize_current_settings},
+            },
+            setup_pool,
+        },
+    };
+    use defguard_mail::Mail;
+    use sqlx::postgres::{PgConnectOptions, PgPoolOptions};
+    use tokio::sync::mpsc::unbounded_channel;
+
+    use crate::db::{
+        User,
+        models::enrollment::{ENROLLMENT_TOKEN_TYPE, Token},
+    };
+
+    #[sqlx::test]
+    async fn dg25_11_test_enrollment_welcome_email(_: PgPoolOptions, options: PgConnectOptions) {
+        let pool = setup_pool(options).await;
+
+        // initialize server config
+        SERVER_CONFIG
+            .set(DefGuardConfig::new_test_config())
+            .unwrap();
+
+        // setup mail channel
+        let (mail_tx, mut mail_rx) = unbounded_channel::<Mail>();
+
+        // setup users
+        let admin = User::new(
+            "test_admin",
+            Some("pass123"),
+            "Test",
+            "Admin",
+            "admin@test.com",
+            None,
+        )
+        .save(&pool)
+        .await
+        .unwrap();
+        let user = User::new(
+            "test_user",
+            Some("pass123"),
+            "Test",
+            "User",
+            "user@test.com",
+            None,
+        )
+        .save(&pool)
+        .await
+        .unwrap();
+
+        // generate enrollment token
+        let token = Token::new(
+            user.id,
+            Some(admin.id),
+            Some(user.email.clone()),
+            10,
+            Some(ENROLLMENT_TOKEN_TYPE.to_string()),
+        );
+
+        // initialize settings
+        Settings::init_defaults(&pool).await.unwrap();
+        initialize_current_settings(&pool).await.unwrap();
+
+        let mut settings = Settings::get(&pool).await.unwrap().unwrap();
+
+        // send welcome email
+        let mut transaction = pool.begin().await.unwrap();
+        token
+            .send_welcome_email(
+                &mut transaction,
+                &mail_tx,
+                &user,
+                &settings,
+                "127.0.0.1",
+                None,
+            )
+            .await
+            .unwrap();
+
+        // check email content
+        let mail = mail_rx.recv().await.unwrap();
+        assert_eq!(mail.to, user.email);
+        assert_eq!(
+            mail.subject,
+            settings.enrollment_welcome_email_subject.unwrap()
+        );
+
+        // set subject to None
+        settings.enrollment_welcome_email_subject = None;
+
+        // send another welcome email
+        let mut transaction = pool.begin().await.unwrap();
+        token
+            .send_welcome_email(
+                &mut transaction,
+                &mail_tx,
+                &user,
+                &settings,
+                "127.0.0.1",
+                None,
+            )
+            .await
+            .unwrap();
+
+        // check email content
+        let mail = mail_rx.recv().await.unwrap();
+        assert_eq!(mail.to, user.email);
+        assert_eq!(mail.subject, WELCOME_EMAIL_SUBJECT);
+    }
+}
