@@ -8,6 +8,8 @@ use axum::{
     http::StatusCode,
 };
 use chrono::NaiveDateTime;
+use defguard_common::{csv::AsCsv, db::Id};
+use defguard_mail::templates::TemplateLocation;
 use ipnetwork::IpNetwork;
 use serde_json::json;
 use sqlx::PgConnection;
@@ -17,18 +19,16 @@ use crate::{
     appstate::AppState,
     auth::{AdminRole, SessionInfo},
     db::{
+        Device, GatewayEvent, User, WireguardNetwork,
         models::{
             device::{DeviceConfig, DeviceInfo, DeviceType, WireguardNetworkDevice},
             wireguard::NetworkAddressError,
         },
-        Device, GatewayEvent, Id, User, WireguardNetwork,
     },
     enterprise::limits::update_counts,
     events::{ApiEvent, ApiEventType, ApiRequestContext},
     handlers::mail::send_new_device_added_email,
     server_config,
-    templates::TemplateLocation,
-    AsCsv,
 };
 
 #[derive(Serialize)]
@@ -163,7 +163,9 @@ pub async fn get_network_device(
             });
         }
     }
-    error!("Failed to retrieve network device with id: {device_id}, such network device doesn't exist.");
+    error!(
+        "Failed to retrieve network device with id: {device_id}, such network device doesn't exist."
+    );
     Err(WebError::ObjectNotFound(format!(
         "Network device with ID {device_id} not found"
     )))
@@ -219,6 +221,19 @@ pub struct IpAvailabilityCheck {
     ips: Vec<String>,
 }
 
+#[derive(Serialize)]
+pub struct IpAvailabilityCheckResult {
+    available: bool,
+    valid: bool,
+}
+
+impl IpAvailabilityCheckResult {
+    #[must_use]
+    pub fn new(available: bool, valid: bool) -> Self {
+        Self { available, valid }
+    }
+}
+
 pub(crate) async fn check_ip_availability(
     _admin_role: AdminRole,
     Path(network_id): Path<i64>,
@@ -226,71 +241,76 @@ pub(crate) async fn check_ip_availability(
     Json(check): Json<IpAvailabilityCheck>,
 ) -> ApiResult {
     let mut transaction = appstate.pool.begin().await?;
-    let network = WireguardNetwork::find_by_id(&appstate.pool, network_id)
+
+    // fetch relevant WireGuard location
+    let location = WireguardNetwork::find_by_id(&appstate.pool, network_id)
         .await?
         .ok_or_else(|| {
             error!(
-                "Failed to check IP availability for network with ID {network_id}, network not found",
+                "Failed to check IP availability for location with ID {network_id}, location not found",
 
             );
-            WebError::BadRequest("Failed to check IP availability, network not found".into())
+            WebError::BadRequest("Failed to check IP availability, location not found".into())
         })?;
-    let ips = check
-        .ips
-        .iter()
-        .map(|ip| IpAddr::from_str(ip))
-        .collect::<Result<Vec<IpAddr>, AddrParseError>>();
-    let Ok(ips) = ips else {
-        warn!(
-            "Failed to check IP availability for network {}, invalid IP address",
-            network.name
-        );
-        return Ok(ApiResponse {
-            json: json!({
-                "available": false,
-                "valid": false,
-            }),
-            status: StatusCode::OK,
-        });
-    };
 
-    let mkresponse = |available: bool, valid: bool| {
-        Ok(ApiResponse {
-            json: json!({
-                "available": available,
-                "valid": valid,
-            }),
-            status: StatusCode::OK,
-        })
-    };
-    return match network.can_assign_ips(&mut transaction, &ips, None).await {
-        Ok(_) => mkresponse(true, true),
-        Err(NetworkAddressError::NoContainingNetwork(name, ip, networks)) => {
-            warn!(
-                "Provided device IP address {ip} is not in the network {name} range: {networks:?}"
-            );
-            mkresponse(false, false)
+    // process IPs one by one and preserve order in response
+    let mut validation_results = Vec::new();
+    for ip in &check.ips {
+        match IpAddr::from_str(ip) {
+            Ok(ip) => {
+                debug!(
+                    "Checking if IP address {ip} can be assigned to a device in location {location}",
+                );
+                let result = match location.can_assign_ips(&mut transaction, &[ip], None).await {
+                    Ok(()) => IpAvailabilityCheckResult::new(true, true),
+                    Err(NetworkAddressError::NoContainingNetwork(name, ip, networks)) => {
+                        warn!(
+                            "Provided device IP address {ip} is not in the network {name} range: {networks:?}"
+                        );
+                        IpAvailabilityCheckResult::new(false, false)
+                    }
+                    Err(NetworkAddressError::ReservedForGateway(name, ip)) => {
+                        warn!(
+                            "Provided device IP address {ip} may overlap with the gateway's IP address on network {name}",
+                        );
+                        IpAvailabilityCheckResult::new(false, true)
+                    }
+                    Err(NetworkAddressError::IsBroadcastAddress(name, ip)) => {
+                        warn!(
+                            "Provided device IP address {ip} is broadcast address of network {name}"
+                        );
+                        IpAvailabilityCheckResult::new(false, true)
+                    }
+                    Err(NetworkAddressError::IsNetworkAddress(name, ip)) => {
+                        warn!(
+                            "Provided device IP address {ip} is network address of network {name}"
+                        );
+                        IpAvailabilityCheckResult::new(false, true)
+                    }
+                    Err(NetworkAddressError::AddressAlreadyAssigned(name, ip)) => {
+                        warn!("Provided device IP {ip} is already assigned in network {name}");
+                        IpAvailabilityCheckResult::new(false, true)
+                    }
+                    Err(NetworkAddressError::DbError(err)) => Err(err)?,
+                };
+                validation_results.push(result);
+            }
+            Err(_err) => {
+                warn!(
+                    "Failed to check IP availability for location {location}, invalid IP address {ip}",
+                );
+                validation_results.push(IpAvailabilityCheckResult {
+                    available: false,
+                    valid: false,
+                });
+            }
         }
-        Err(NetworkAddressError::ReservedForGateway(name, ip)) => {
-            warn!(
-                "Provided device IP address {ip} may overlap with the gateway's IP address on network {name}",
-            );
-            mkresponse(false, true)
-        }
-        Err(NetworkAddressError::IsBroadcastAddress(name, ip)) => {
-            warn!("Provided device IP address {ip} is broadcast address of network {name}");
-            mkresponse(false, true)
-        }
-        Err(NetworkAddressError::IsNetworkAddress(name, ip)) => {
-            warn!("Provided device IP address {ip} is network address of network {name}");
-            mkresponse(false, true)
-        }
-        Err(NetworkAddressError::AddressAlreadyAssigned(name, ip)) => {
-            warn!("Provided device IP {ip} is already assigned in network {name}");
-            mkresponse(false, true)
-        }
-        Err(NetworkAddressError::DbError(err)) => Err(err)?,
-    };
+    }
+
+    Ok(ApiResponse {
+        json: json!(validation_results),
+        status: StatusCode::OK,
+    })
 }
 
 pub(crate) async fn find_available_ips(
@@ -331,16 +351,7 @@ pub(crate) async fn find_available_ips(
     }
 
     transaction.commit().await?;
-    if split_ips.len() != network.address.len() {
-        warn!(
-            "Failed to find available IPs for new device in network {} ({:?})",
-            network.name, network.address
-        );
-        Ok(ApiResponse {
-            json: json!({}),
-            status: StatusCode::NOT_FOUND,
-        })
-    } else {
+    if split_ips.len() == network.address.len() {
         debug!(
             "Found addresses {:?} for new device i network {} ({:?})",
             split_ips, network.name, network.address
@@ -349,6 +360,15 @@ pub(crate) async fn find_available_ips(
             json: json!(split_ips),
             status: StatusCode::OK,
         })
+    } else {
+        warn!(
+            "Failed to find available IPs for new device in network {} ({:?})",
+            network.name, network.address
+        );
+        Ok(ApiResponse {
+            json: json!({}),
+            status: StatusCode::NOT_FOUND,
+        })
     }
 }
 
@@ -631,27 +651,25 @@ pub(crate) async fn add_network_device(
         session.session.device_info.clone().as_deref(),
     )?;
 
+    let result = AddNetworkDeviceResult {
+        config,
+        device: NetworkDeviceInfo::from_device(device.clone(), &mut transaction).await?,
+    };
+
+    transaction.commit().await?;
+
     info!(
         "User {} added a new network device {device_name}.",
         user.username
     );
     appstate.emit_event(ApiEvent {
         context,
-        event: ApiEventType::NetworkDeviceAdded {
-            device_id: device.id,
-            device_name: device.name.clone(),
-            location_id: network.id,
-            location: network.name,
-        },
+        event: Box::new(ApiEventType::NetworkDeviceAdded {
+            device,
+            location: network,
+        }),
     })?;
 
-    let result = AddNetworkDeviceResult {
-        config,
-        device: NetworkDeviceInfo::from_device(device, &mut transaction).await?,
-    };
-
-    transaction.commit().await?;
-
     Ok(ApiResponse {
         json: json!(result),
         status: StatusCode::CREATED,
@@ -681,6 +699,8 @@ pub async fn modify_network_device(
             error!("Failed to update device {device_id}, device not found");
             WebError::ObjectNotFound(format!("Device {device_id} not found"))
         })?;
+    // store device before modifications
+    let before = device.clone();
     let device_network = device
         .find_network_device_networks(&mut *transaction)
         .await?
@@ -733,19 +753,18 @@ pub async fn modify_network_device(
             device_network.name
         );
     }
-
-    let network_device_info = NetworkDeviceInfo::from_device(device, &mut transaction).await?;
-    appstate.emit_event(ApiEvent {
-        context,
-        event: ApiEventType::NetworkDeviceModified {
-            device_id: network_device_info.id,
-            device_name: network_device_info.name.clone(),
-            location_id: device_network.id,
-            location: device_network.name,
-        },
-    })?;
+    let network_device_info =
+        NetworkDeviceInfo::from_device(device.clone(), &mut transaction).await?;
     transaction.commit().await?;
 
+    appstate.emit_event(ApiEvent {
+        context,
+        event: Box::new(ApiEventType::NetworkDeviceModified {
+            location: device_network,
+            before,
+            after: device,
+        }),
+    })?;
     Ok(ApiResponse {
         json: json!(network_device_info),
         status: StatusCode::OK,
