@@ -4,18 +4,25 @@ use std::{
 };
 
 use axum::{
+    Form,
     extract::{FromRef, OptionalFromRequestParts, Query, State},
     http::{
+        HeaderMap, HeaderValue, StatusCode,
         header::{AUTHORIZATION, LOCATION},
         request::Parts,
-        HeaderMap, HeaderValue, StatusCode,
     },
-    Form,
 };
 use axum_extra::extract::cookie::{Cookie, CookieJar, PrivateCookieJar, SameSite};
-use base64::{prelude::BASE64_STANDARD, Engine};
+use base64::{Engine, prelude::BASE64_STANDARD};
 use chrono::Utc;
+use defguard_common::db::{Id, NoId, models::AuthCode};
 use openidconnect::{
+    AccessToken, AdditionalClaims, Audience, AuthUrl, AuthorizationCode,
+    EmptyAdditionalProviderMetadata, EmptyExtraTokenFields, EndUserEmail, EndUserFamilyName,
+    EndUserGivenName, EndUserName, EndUserPhoneNumber, EndUserUsername, IdToken, IdTokenClaims,
+    IdTokenFields, IssuerUrl, JsonWebKeySetUrl, LocalizedClaim, Nonce, PkceCodeChallenge,
+    PkceCodeVerifier, PrivateSigningKey, RefreshToken, ResponseTypes, Scope, StandardClaims,
+    StandardErrorResponse, StandardTokenResponse, SubjectIdentifier, TokenUrl, UserInfoUrl,
     core::{
         CoreAuthErrorResponseType, CoreClaimName, CoreErrorResponseType, CoreGenderClaim,
         CoreGrantType, CoreHmacKey, CoreJsonWebKeySet, CoreJweContentEncryptionAlgorithm,
@@ -23,12 +30,6 @@ use openidconnect::{
         CoreSubjectIdentifierType, CoreTokenType,
     },
     url::Url,
-    AccessToken, AdditionalClaims, Audience, AuthUrl, AuthorizationCode,
-    EmptyAdditionalProviderMetadata, EmptyExtraTokenFields, EndUserEmail, EndUserFamilyName,
-    EndUserGivenName, EndUserName, EndUserPhoneNumber, EndUserUsername, IdToken, IdTokenClaims,
-    IdTokenFields, IssuerUrl, JsonWebKeySetUrl, LocalizedClaim, Nonce, PkceCodeChallenge,
-    PkceCodeVerifier, PrivateSigningKey, RefreshToken, ResponseTypes, Scope, StandardClaims,
-    StandardErrorResponse, StandardTokenResponse, SubjectIdentifier, TokenUrl, UserInfoUrl,
 };
 use serde::{
     de::{Deserialize, Deserializer, Error as DeError, Unexpected, Visitor},
@@ -41,38 +42,52 @@ use time::Duration;
 use super::{ApiResponse, ApiResult, SESSION_COOKIE_NAME};
 use crate::{
     appstate::AppState,
-    auth::{AccessUserInfo, SessionInfo},
+    auth::{SessionInfo, UserClaims},
     db::{
-        models::{auth_code::AuthCode, oauth2client::OAuth2Client},
-        Id, OAuth2AuthorizedApp, OAuth2Token, Session, SessionState, User,
+        OAuth2AuthorizedApp, OAuth2Token, Session, SessionState, User,
+        models::oauth2client::OAuth2Client,
     },
     error::WebError,
-    handlers::{mail::send_new_device_ocid_login_email, SIGN_IN_COOKIE_NAME},
+    handlers::{SIGN_IN_COOKIE_NAME, mail::send_new_device_ocid_login_email},
     server_config,
 };
 
 /// https://openid.net/specs/openid-connect-core-1_0.html#StandardClaims
-impl From<&User<Id>> for StandardClaims<CoreGenderClaim> {
-    fn from(user: &User<Id>) -> StandardClaims<CoreGenderClaim> {
-        let mut name = LocalizedClaim::new();
-        name.insert(None, EndUserName::new(user.name()));
-        let mut given_name = LocalizedClaim::new();
-        given_name.insert(None, EndUserGivenName::new(user.first_name.clone()));
-        let mut given_name = LocalizedClaim::new();
-        given_name.insert(None, EndUserGivenName::new(user.first_name.clone()));
-        let mut family_name = LocalizedClaim::new();
-        family_name.insert(None, EndUserFamilyName::new(user.last_name.clone()));
-        let email = EndUserEmail::new(user.email.clone());
-        let phone_number = user.phone.clone().map(EndUserPhoneNumber::new);
-        let preferred_username = EndUserUsername::new(user.username.clone());
+impl From<&UserClaims> for StandardClaims<CoreGenderClaim> {
+    fn from(user_claims: &UserClaims) -> StandardClaims<CoreGenderClaim> {
+        let mut claims = StandardClaims::new(SubjectIdentifier::new(user_claims.sub.clone()));
 
-        StandardClaims::new(SubjectIdentifier::new(user.username.clone()))
-            .set_name(Some(name))
-            .set_given_name(Some(given_name))
-            .set_family_name(Some(family_name))
-            .set_email(Some(email))
-            .set_phone_number(phone_number)
-            .set_preferred_username(Some(preferred_username))
+        if let Some(name) = &user_claims.name {
+            let mut localized_claim = LocalizedClaim::new();
+            localized_claim.insert(None, EndUserName::new(name.clone()));
+            claims = claims.set_name(Some(localized_claim));
+        }
+
+        if let Some(given_name) = &user_claims.given_name {
+            let mut localized_claim = LocalizedClaim::new();
+            localized_claim.insert(None, EndUserGivenName::new(given_name.clone()));
+            claims = claims.set_given_name(Some(localized_claim));
+        }
+
+        if let Some(family_name) = &user_claims.family_name {
+            let mut localized_claim = LocalizedClaim::new();
+            localized_claim.insert(None, EndUserFamilyName::new(family_name.clone()));
+            claims = claims.set_family_name(Some(localized_claim));
+        }
+
+        if let Some(email) = &user_claims.email {
+            claims = claims.set_email(Some(EndUserEmail::new(email.clone())));
+        }
+
+        if let Some(phone_number) = &user_claims.phone_number {
+            claims = claims.set_phone_number(Some(EndUserPhoneNumber::new(phone_number.clone())));
+        }
+
+        if let Some(username) = &user_claims.preferred_username {
+            claims = claims.set_preferred_username(Some(EndUserUsername::new(username.clone())));
+        }
+
+        claims
     }
 }
 
@@ -257,18 +272,7 @@ impl AuthenticationRequest {
 
         // assume `client_id` is the same here and in `oauth2client`
 
-        // check `redirect_uri` matches client config (ignoring trailing slashes)
-        let parsed_redirect_uris: Vec<String> = oauth2client
-            .redirect_uri
-            .iter()
-            .map(|uri| uri.trim_end_matches('/').into())
-            .collect();
-        if self
-            .redirect_uri
-            .split(' ')
-            .map(|uri| uri.trim_end_matches('/'))
-            .all(|uri| !parsed_redirect_uris.iter().any(|u| u == uri))
-        {
+        if !oauth2client.contains_redirect_url(&self.redirect_uri) {
             error!(
                 "Invalid redirect_uri for client {}: {} not in [{}]",
                 oauth2client.name,
@@ -321,7 +325,7 @@ async fn generate_auth_code_redirect(
         if let Some(state) = data.state {
             query_pairs.append_pair("state", &state);
         }
-    };
+    }
 
     Ok(url.to_string())
 }
@@ -378,11 +382,16 @@ pub async fn authorization(
     private_cookies: PrivateCookieJar,
 ) -> Result<(StatusCode, HeaderMap, PrivateCookieJar), WebError> {
     let error;
+    let mut is_redirect_allowed = false;
     if let Some(oauth2client) =
         OAuth2Client::find_by_client_id(&appstate.pool, &data.client_id).await?
     {
-        match data.validate_for_client(&oauth2client) {
-            Ok(()) => {
+        is_redirect_allowed = oauth2client.contains_redirect_url(&data.redirect_uri);
+        match (
+            oauth2client.enabled,
+            data.validate_for_client(&oauth2client),
+        ) {
+            (true, Ok(())) => {
                 match &data.prompt {
                     Some(s) if s == "consent" => {
                         info!(
@@ -391,7 +400,7 @@ pub async fn authorization(
                         );
                         // FIXME: do not panic
                         return Ok(redirect_to(
-                            format!("/consent?{}", serde_urlencoded::to_string(data).unwrap(),),
+                            format!("/consent?{}", serde_urlencoded::to_string(data).unwrap()),
                             private_cookies,
                         ));
                     }
@@ -406,7 +415,11 @@ pub async fn authorization(
                             {
                                 // If session expired return login
                                 if session.expired() {
-                                    info!("Session {} for user id {} has expired, redirecting to login", session.id, session.user_id);
+                                    info!(
+                                        "Session {} for user id {} has expired, redirecting to \
+                                        login",
+                                        session.id, session.user_id
+                                    );
                                     let _result = session.delete(&appstate.pool).await;
                                     Ok(login_redirect(&data, private_cookies))
                                 } else {
@@ -419,8 +432,9 @@ pub async fn authorization(
 
                                     user.verify_mfa_state(&appstate.pool).await?;
 
-                                    // Session exists even if user hasn't completed MFA verification yet,
-                                    // thus we need to check if MFA is enabled and the verification is done.
+                                    // Session exists even if user hasn't completed MFA verification
+                                    // yet, thus we need to check if MFA is enabled and the
+                                    // verification is done.
                                     if user.mfa_enabled
                                         && session.state != SessionState::MultiFactorVerified
                                     {
@@ -431,8 +445,9 @@ pub async fn authorization(
                                         return Ok(login_redirect(&data, private_cookies));
                                     }
 
-                                    // If session is present check if app is in user authorized apps.
-                                    // If yes return auth code and state else redirect to consent form.
+                                    // If session is present check if app is in user authorized
+                                    // apps. If yes, return auth code and state else redirect to
+                                    // consent form.
                                     if let Some(app) =
                                         OAuth2AuthorizedApp::find_by_user_and_oauth2client_id(
                                             &appstate.pool,
@@ -442,7 +457,8 @@ pub async fn authorization(
                                         .await?
                                     {
                                         info!(
-                                            "OAuth client id {} authorized by user id {}, returning auth code",
+                                            "OAuth client id {} authorized by user id {}, \
+                                            returning auth code",
                                             app.oauth2client_id, session.user_id
                                         );
                                         let private_cookies = private_cookies
@@ -457,7 +473,8 @@ pub async fn authorization(
                                     } else {
                                         // If authorized app not found redirect to consent form
                                         info!(
-                                            "OAuth client id {} not yet authorized by user id {}, redirecting to consent form",
+                                            "OAuth client id {} not yet authorized by user id {}, \
+                                            redirecting to consent form",
                                             oauth2client.id, session.user_id
                                         );
                                         Ok(redirect_to(
@@ -470,14 +487,13 @@ pub async fn authorization(
                                     }
                                 }
                             } else {
-                                // If session is not present in db redirect to login
+                                // If session is not present in database, redirect to login.
                                 info!(
                                     "Session {} not found, redirecting to login page",
                                     session_cookie.value()
                                 );
                                 Ok(login_redirect(&data, private_cookies))
                             }
-
                         // If no session cookie provided redirect to login
                         } else {
                             info!("Session cookie not provided, redirecting to login page");
@@ -486,22 +502,29 @@ pub async fn authorization(
                     }
                 }
             }
-            Err(err) => {
+            (true, Err(err)) => {
                 error!(
                     "OIDC login validation failed for client {}: {err:?}",
                     data.client_id
                 );
                 error = err;
             }
+            (false, _) => {
+                error!("OIDC client id {} is disabled", data.client_id);
+                error = CoreAuthErrorResponseType::UnauthorizedClient;
+            }
         }
     } else {
         error!("OIDC client id {} not found", data.client_id);
         error = CoreAuthErrorResponseType::UnauthorizedClient;
     }
 
-    let mut url =
-        Url::parse(&data.redirect_uri).map_err(|_| WebError::Http(StatusCode::BAD_REQUEST))?;
-
+    let mut url = if is_redirect_allowed {
+        Url::parse(&data.redirect_uri).map_err(|_| WebError::Http(StatusCode::BAD_REQUEST))?
+    } else {
+        // Don't allow open redirects (DG25-17)
+        server_config().url.clone()
+    };
     {
         let mut query_pairs = url.query_pairs_mut();
         query_pairs.append_pair("error", error.as_ref());
@@ -521,7 +544,7 @@ pub struct GroupClaims {
 
 impl AdditionalClaims for GroupClaims {}
 
-pub async fn get_group_claims(pool: &PgPool, user: &User<Id>) -> Result<GroupClaims, WebError> {
+async fn get_group_claims(pool: &PgPool, user: &User<Id>) -> Result<GroupClaims, WebError> {
     let groups = user.member_of_names(pool).await?;
     Ok(GroupClaims {
         groups: Some(groups),
@@ -535,15 +558,18 @@ pub async fn secure_authorization(
     Query(data): Query<AuthenticationRequest>,
     private_cookies: PrivateCookieJar,
 ) -> Result<(StatusCode, HeaderMap, PrivateCookieJar), WebError> {
-    let mut url =
-        Url::parse(&data.redirect_uri).map_err(|_| WebError::Http(StatusCode::BAD_REQUEST))?;
     let error;
-    if data.allow {
-        if let Some(oauth2client) =
-            OAuth2Client::find_by_client_id(&appstate.pool, &data.client_id).await?
-        {
-            match data.validate_for_client(&oauth2client) {
-                Ok(()) => {
+    let mut is_redirect_allowed = false;
+    if let Some(oauth2client) =
+        OAuth2Client::find_by_client_id(&appstate.pool, &data.client_id).await?
+    {
+        is_redirect_allowed = oauth2client.contains_redirect_url(&data.redirect_uri);
+        if data.allow {
+            match (
+                oauth2client.enabled,
+                data.validate_for_client(&oauth2client),
+            ) {
+                (true, Ok(())) => {
                     if OAuth2AuthorizedApp::find_by_user_and_oauth2client_id(
                         &appstate.pool,
                         session_info.user.id,
@@ -559,7 +585,7 @@ pub async fn secure_authorization(
                             &session_info.user.email,
                             oauth2client.name.to_string(),
                             &appstate.mail_tx,
-                            &session_info.session,
+                            &session_info.session.into(),
                         )
                         .await?;
                     }
@@ -576,29 +602,39 @@ pub async fn secure_authorization(
                     );
                     return Ok(redirect_to(location, private_cookies));
                 }
-                Err(err) => {
+                (true, Err(err)) => {
                     info!(
                         "OIDC login validation failed for user {}, client {}",
                         session_info.user.username, oauth2client.name
                     );
                     error = err;
                 }
+                (false, _) => {
+                    error!("OIDC client id {} is disabled", oauth2client.name);
+                    error = CoreAuthErrorResponseType::UnauthorizedClient;
+                }
             }
         } else {
-            error!(
-                "User {} tried to log in with non-existent OIDC client id {}",
+            info!(
+                "User {} denied OIDC login with app id {}",
                 session_info.user.username, data.client_id
             );
-            error = CoreAuthErrorResponseType::UnauthorizedClient;
+            error = CoreAuthErrorResponseType::AccessDenied;
         }
     } else {
-        info!(
-            "User {} denied OIDC login with app id {}",
+        error!(
+            "User {} tried to log in with non-existent OIDC client id {}",
             session_info.user.username, data.client_id
         );
-        error = CoreAuthErrorResponseType::AccessDenied;
+        error = CoreAuthErrorResponseType::UnauthorizedClient;
     }
 
+    let mut url = if is_redirect_allowed {
+        Url::parse(&data.redirect_uri).map_err(|_| WebError::Http(StatusCode::BAD_REQUEST))?
+    } else {
+        // Don't allow open redirects (DG25-17)
+        server_config().url.clone()
+    };
     {
         let mut query_pairs = url.query_pairs_mut();
         query_pairs.append_pair("error", error.as_ref());
@@ -646,7 +682,7 @@ impl TokenRequest {
 
     fn authorization_code_flow<T>(
         &self,
-        auth_code: &AuthCode<Id>,
+        auth_code: &AuthCode<NoId>,
         token: &OAuth2Token,
         claims: StandardClaims<CoreGenderClaim>,
         base_url: &Url,
@@ -777,21 +813,30 @@ pub async fn token(
 
             // for logging
             let form_client_id = match &form.client_id {
-                Some(id) => id.clone(),
-                None => String::from("N/A"),
+                Some(id) => id,
+                None => "N/A",
             };
 
             if let Some(code) = &form.code {
-                if let Some(stored_auth_code) = AuthCode::find_code(&appstate.pool, code).await? {
-                    // copy data before removing used token
-                    let auth_code = stored_auth_code.clone();
-                    // remove authorization_code from DB so it cannot be reused
-                    debug!(
-                        "Removing used authorization_code {code}, client_id `{}`",
-                        form_client_id
-                    );
-                    stored_auth_code.consume(&appstate.pool).await?;
+                // Look for `AuthCode`. If found, it will be deleted from the database to avoid
+                // concurrent requests that might return multiple tokens for the same code.
+                // This addresses DG25-24 and conforms to RFC 6749.
+                if let Some(auth_code) = AuthCode::find_code(&appstate.pool, code).await? {
+                    debug!("Consumed authorization_code {code}, client_id `{form_client_id}`");
                     if let Some(client) = oauth2client.or(form.oauth2client(&appstate.pool).await) {
+                        if !client.enabled {
+                            error!("OAuth client id `{}` is disabled", client.name);
+                            let response = StandardErrorResponse::<CoreErrorResponseType>::new(
+                                CoreErrorResponseType::UnauthorizedClient,
+                                None,
+                                None,
+                            );
+                            return Ok(ApiResponse {
+                                json: json!(response),
+                                status: StatusCode::BAD_REQUEST,
+                            });
+                        }
+
                         if let Some(user) =
                             User::find_by_id(&appstate.pool, auth_code.user_id).await?
                         {
@@ -807,7 +852,7 @@ pub async fn token(
                                     "Issuing new token for user {} client {}",
                                     user.username, client.name
                                 );
-                                // Remove existing token in case same client asks for new token
+                                // Remove existing token in case the same client asks for new token.
                                 if let Some(token) = OAuth2Token::find_by_authorized_app_id(
                                     &appstate.pool,
                                     authorized_app.id,
@@ -827,10 +872,11 @@ pub async fn token(
                                     GroupClaims { groups: None }
                                 };
                                 let config = server_config();
+                                let user_claims = UserClaims::from_user(&user, &client, &token);
                                 match form.authorization_code_flow(
                                     &auth_code,
                                     &token,
-                                    (&user).into(),
+                                    (&user_claims).into(),
                                     &config.url,
                                     client.client_secret,
                                     config.openid_key(),
@@ -849,8 +895,8 @@ pub async fn token(
                                     }
                                     Err(err) => {
                                         error!(
-                                            "Error issuing new token for user {} client {}: {}",
-                                            user.username, client.name, err
+                                            "Error issuing new token for user {} client {}: {err}",
+                                            user.username, client.name
                                         );
                                         let response =
                                             StandardErrorResponse::<CoreErrorResponseType>::new(
@@ -863,7 +909,11 @@ pub async fn token(
                                     }
                                 }
                             }
-                            error!("Can't issue token - authorized app not found for user {}, client {}", user.username, client.name);
+                            error!(
+                                "Can't issue token - authorized app not found for user {}, client \
+                                {}",
+                                user.username, client.name
+                            );
                         } else {
                             error!("User id {} not found", auth_code.user_id);
                         }
@@ -874,7 +924,7 @@ pub async fn token(
                     error!("OAuth auth code not found");
                 }
             } else {
-                error!("No code provided in request for client id `{form_client_id}`",);
+                error!("No code provided in request for client id `{form_client_id}`");
             }
         }
         "refresh_token" => {
@@ -883,6 +933,31 @@ pub async fn token(
                 if let Ok(Some(mut token)) =
                     OAuth2Token::find_refresh_token(&appstate.pool, &refresh_token).await
                 {
+                    let Some(client) = OAuth2Client::find_by_token(&appstate.pool, &token).await?
+                    else {
+                        error!("OAuth client not found for provided refresh_token");
+                        let err = CoreErrorResponseType::InvalidClient;
+                        let response =
+                            StandardErrorResponse::<CoreErrorResponseType>::new(err, None, None);
+                        return Ok(ApiResponse {
+                            json: json!(response),
+                            status: StatusCode::BAD_REQUEST,
+                        });
+                    };
+
+                    if !client.enabled {
+                        error!("OAuth client id `{}` is disabled", client.name);
+                        let response = StandardErrorResponse::<CoreErrorResponseType>::new(
+                            CoreErrorResponseType::UnauthorizedClient,
+                            None,
+                            None,
+                        );
+                        return Ok(ApiResponse {
+                            json: json!(response),
+                            status: StatusCode::BAD_REQUEST,
+                        });
+                    }
+
                     token.refresh_and_save(&appstate.pool).await?;
                     let response = TokenRequest::refresh_token_flow(&token);
                     token.save(&appstate.pool).await?;
@@ -904,10 +979,49 @@ pub async fn token(
 }
 
 /// https://openid.net/specs/openid-connect-core-1_0.html#UserInfo
-pub async fn userinfo(user_info: AccessUserInfo) -> ApiResult {
-    let userclaims = StandardClaims::<CoreGenderClaim>::from(&user_info.0);
+pub async fn userinfo(State(appstate): State<AppState>, headers: HeaderMap) -> ApiResult {
+    let Some(token) = headers.get(AUTHORIZATION).and_then(|value| {
+        if let Ok(value) = value.to_str() {
+            if value.to_lowercase().starts_with("bearer ") {
+                value.get(7..)
+            } else {
+                None
+            }
+        } else {
+            None
+        }
+    }) else {
+        return Err(WebError::Authorization("Invalid session".into()));
+    };
+
+    let Some(oauth2token) = OAuth2Token::find_access_token(&appstate.pool, token).await? else {
+        return Err(WebError::Authorization("Invalid token".into()));
+    };
+
+    let Some(authorized_app) =
+        OAuth2AuthorizedApp::find_by_id(&appstate.pool, oauth2token.oauth2authorizedapp_id).await?
+    else {
+        return Err(WebError::Authorization("Authorized app not found".into()));
+    };
+
+    let Some(client) =
+        OAuth2Client::find_by_id(&appstate.pool, authorized_app.oauth2client_id).await?
+    else {
+        return Err(WebError::Authorization("OAuth2 client not found".into()));
+    };
+
+    if !client.enabled {
+        return Err(WebError::Authorization("OAuth2 client is disabled".into()));
+    }
+
+    let Some(user) = User::find_by_id(&appstate.pool, authorized_app.user_id).await? else {
+        return Err(WebError::Authorization("User not found".into()));
+    };
+
+    let user_claims = UserClaims::from_user(&user, &client, &oauth2token);
+
     Ok(ApiResponse {
-        json: json!(userclaims),
+        json: json!(StandardClaims::<CoreGenderClaim>::from(&user_claims)),
         status: StatusCode::OK,
     })
 }
