@@ -4,24 +4,26 @@ use axum::{
     extract::{Json, Path, State},
     http::StatusCode,
 };
+use defguard_mail::{Mail, templates};
 use serde_json::json;
 
 use super::{
-    mail::EMAIL_PASSOWRD_RESET_START_SUBJECT, user_for_admin_or_self, AddUserData, ApiResponse,
-    ApiResult, PasswordChange, PasswordChangeSelf, StartEnrollmentRequest, Username,
+    AddUserData, ApiResponse, ApiResult, PasswordChange, PasswordChangeSelf,
+    StartEnrollmentRequest, Username, mail::EMAIL_PASSWORD_RESET_START_SUBJECT,
+    user_for_admin_or_self,
 };
 use crate::{
     appstate::AppState,
     auth::{AdminRole, SessionInfo},
     db::{
-        models::{
-            enrollment::{Token, PASSWORD_RESET_TOKEN_TYPE},
-            GroupDiff,
-        },
         AppEvent, OAuth2AuthorizedApp, User, UserDetails, UserInfo, WebAuthn,
+        models::{
+            GroupDiff,
+            enrollment::{PASSWORD_RESET_TOKEN_TYPE, Token},
+        },
     },
     enterprise::{
-        db::models::enterprise_settings::EnterpriseSettings,
+        db::models::{api_tokens::ApiToken, enterprise_settings::EnterpriseSettings},
         ldap::utils::{
             ldap_add_user, ldap_add_user_to_groups, ldap_change_password, ldap_delete_user,
             ldap_handle_user_modify, ldap_remove_user_from_groups, ldap_update_user_state,
@@ -30,15 +32,18 @@ use crate::{
     },
     error::WebError,
     events::{ApiEvent, ApiEventType, ApiRequestContext},
-    mail::Mail,
-    server_config, templates,
+    is_valid_phone_number, server_config,
 };
 
+/// The maximum length for the commonName (CN) attribute in LDAP schemas is commonly set to 64
+/// characters according to the X.520 standard and many LDAP implementations like Active Directory.
+pub(crate) const MAX_USERNAME_CHARS: usize = 64;
+
 /// Verify the given username
 ///
 /// To enable LDAP sync usernames need to avoid reserved characters.
 /// Username requirements:
-/// - 1 - 64 characters long
+/// - 1 - MAX_USERNAME_CHARS characters long
 /// - lowercase or uppercase latin alphabet letters (A-Z, a-z)
 /// - digits (0-9)
 /// - starts with non-special character
@@ -47,7 +52,7 @@ use crate::{
 pub fn check_username(username: &str) -> Result<(), WebError> {
     // check length
     let length = username.len();
-    if !(1..64).contains(&length) {
+    if !(1..MAX_USERNAME_CHARS).contains(&length) {
         return Err(WebError::Serialization(format!(
             "Username ({username}) has incorrect length"
         )));
@@ -102,10 +107,12 @@ pub(crate) fn check_password_strength(password: &str) -> Result<(), WebError> {
 
 /// List of all users
 ///
-/// Retrives list of users.
+/// Retrieves list of users.
 ///
 /// # Returns
-/// Returns list of `UserInfo` objects or `WebError` if error occurs.
+/// - List of `UserInfo` objects.
+///
+/// - `WebError` if error occurs
 #[utoipa::path(
     get,
     path = "/api/v1/user",
@@ -113,22 +120,24 @@ pub(crate) fn check_password_strength(password: &str) -> Result<(), WebError> {
         (status = 200, description = "List of all users.", body = [UserInfo], example = json!(
         [
             {
-                "authorized_apps": [],
-                "email": "name@email.com",
+              "authorized_apps": [],
+                "email": "mail@mail",
                 "email_mfa_enabled": false,
                 "enrolled": true,
                 "first_name": "first_name",
                 "groups": [
-                    "group"
+                  "admin"
                 ],
                 "id": 1,
                 "is_active": true,
+                "is_admin": true,
                 "last_name": "last_name",
+                "ldap_pass_requires_change": false,
                 "mfa_enabled": false,
                 "mfa_method": "None",
                 "phone": null,
                 "totp_enabled": false,
-                "username": "username"
+                "username": "admin"
             }
         ])),
         (status = 401, description = "Unauthorized to list all users.", body = ApiResponse, example = json!({"msg": "Session is required"})),
@@ -157,57 +166,39 @@ pub async fn list_users(_role: AdminRole, State(appstate): State<AppState>) -> A
 /// Return a user based on provided username parameter.
 ///
 /// # Returns
-/// Returns `UserDetails` object or `WebError` if error occurs.
+/// - `UserDetails` object
+///
+/// - `WebError` if error occurs
 #[utoipa::path(
     get,
     path = "/api/v1/user/{username}",
     params(
-        ("username" = String, description = "name of a user"),
+        ("username" = String, description = "Name of a user"),
     ),
     responses(
         (status = 200, description = "Return details about user.", body = UserDetails, example = json!(
             {
-                "devices": [
-                    {
-                        "created": "date",
-                        "id": 1,
-                        "name": "name",
-                        "networks": [
-                            {
-                                "device_wireguard_ip": "1.1.1.1",
-                                "is_active": false,
-                                "last_connected_at": null,
-                                "last_connected_ip": null,
-                                "last_connected_location": null,
-                                "network_gateway_ip": "0.0.0.0",
-                                "network_id": 1,
-                                "network_name": "TestNet"
-                            }
-                        ],
-                        "user_id": 1,
-                        "wireguard_pubkey": "wireguard_pubkey"
-                    }
-                ],
-                "security_keys": [],
-                "user": {
-                    "authorized_apps": [],
-                    "email": "name@email.com",
-                    "email_mfa_enabled": false,
-                    "enrolled": true,
-                    "first_name": "first_name",
-                    "groups": [
-                        "group"
-                    ],
-                    "id": 1,
-                    "is_active": true,
-                    "last_name": "last_name",
-                    "mfa_enabled": false,
-                    "mfa_method": "None",
-                    "phone": null,
-                    "totp_enabled": false,
-                    "username": "username"
-                },
-                "wallets": []
+              "biometric_enabled_devices": [],
+              "devices": [],
+              "security_keys": [],
+              "user": {
+                "authorized_apps": [],
+                "email": "mail@defguard.net",
+                "email_mfa_enabled": false,
+                "enrolled": true,
+                "first_name": "first_name",
+                "groups": [],
+                "id": 2,
+                "is_active": true,
+                "is_admin": false,
+                "last_name": "last_name",
+                "ldap_pass_requires_change": false,
+                "mfa_enabled": false,
+                "mfa_method": "None",
+                "phone": "000000000",
+                "totp_enabled": false,
+                "username": "username"
+              }
             }
         )),
         (status = 401, description = "Unauthorized to return details about user.", body = ApiResponse, example = json!({"msg": "Session is required"})),
@@ -237,30 +228,32 @@ pub async fn get_user(
 /// Add a new user based on `AddUserData` object.
 ///
 /// # Returns
-/// Returns `UserInfo` object or `WebError` if error occurs.
+/// - `UserInfo` object
+///
+/// - `WebError` if error occurs
 #[utoipa::path(
     post,
     path = "/api/v1/user",
     request_body = AddUserData,
     responses(
         (status = 201, description = "Add a new user.", body = UserInfo, example = json!(
-            {
-                "authorized_apps": [],
-                "email": "name@email.com",
-                "email_mfa_enabled": false,
-                "enrolled": true,
-                "first_name": "first_name",
-                "groups": [
-                    "admin"
-                ],
-                "id": 1,
-                "is_active": true,
-                "last_name": "last_name",
-                "mfa_enabled": false,
-                "mfa_method": "None",
-                "phone": null,
-                "totp_enabled": false,
-                "username": "username"
+           {
+              "authorized_apps": [],
+              "email": "mail@mail",
+              "email_mfa_enabled": false,
+              "enrolled": true,
+              "first_name": "first_name",
+              "groups": [],
+              "id": 3,
+              "is_active": true,
+              "is_admin": false,
+              "last_name": "last_name",
+              "ldap_pass_requires_change": false,
+              "mfa_enabled": false,
+              "mfa_method": "None",
+              "phone": "000000000",
+              "totp_enabled": false,
+              "username": "new_user"
             }
         )),
         (status = 400, description = "Bad request, invalid user data.", body = ApiResponse, example = json!({})),
@@ -291,6 +284,7 @@ pub async fn add_user(
             status: StatusCode::BAD_REQUEST,
         });
     }
+
     // check if email doesn't already exist
     if User::find_by_email(&appstate.pool, &user_data.email)
         .await?
@@ -302,6 +296,18 @@ pub async fn add_user(
             status: StatusCode::BAD_REQUEST,
         });
     }
+
+    // check phone number
+    if let Some(ref phone) = user_data.phone {
+        if !is_valid_phone_number(phone) {
+            debug!("Invalid phone number for new user {username}: {phone}");
+            return Ok(ApiResponse {
+                json: json!({}),
+                status: StatusCode::BAD_REQUEST,
+            });
+        }
+    }
+
     let password = match &user_data.password {
         Some(password) => {
             // check password strength
@@ -342,9 +348,7 @@ pub async fn add_user(
     }
     appstate.emit_event(ApiEvent {
         context,
-        event: ApiEventType::UserAdded {
-            username: user.username,
-        },
+        event: Box::new(ApiEventType::UserAdded { user }),
     })?;
     Ok(ApiResponse {
         json: json!(&user_info),
@@ -358,11 +362,16 @@ pub async fn add_user(
 ///
 /// Thanks to this endpoint you are able to trigger manually enrollment process, where after finishing you receive an enrollment token.
 ///
-/// `Enrollment token` allows to start the process of gaining access to the company infrastructure `(The enrollment token is valid for 24 hours)`. On the other hand, enrollment url allows the user to access the enrollment form via the web browser or perform the enrollment through the desktop client.
+/// **Enrollment token** allows to start the process of gaining access to the company infrastructure **(The enrollment token is valid for 24 hours)**.
+///
+/// On the other hand, enrollment url allows the user to access the enrollment form via the web browser or perform the enrollment through the desktop client.
 ///
 /// Optionally this endpoint can send an email notification to the user about the enrollment.
+///
 /// # Returns
-/// Returns json with `enrollment token` and `enrollment url` or `WebError` if error occurs.
+/// - JSON with `enrollment_token` and `enrollment_url`
+///
+/// - `WebError` if error occurs
 #[utoipa::path(
     post,
     path = "/api/v1/user/{username}/start_enrollment",
@@ -383,12 +392,13 @@ pub async fn add_user(
 pub async fn start_enrollment(
     _role: AdminRole,
     session: SessionInfo,
+    context: ApiRequestContext,
     State(appstate): State<AppState>,
     Path(username): Path<String>,
     Json(data): Json<StartEnrollmentRequest>,
 ) -> ApiResult {
     debug!(
-        "User {} has started a new enrollment request.",
+        "User {} creating enrollment token for user {username}.",
         session.user.username
     );
 
@@ -435,7 +445,7 @@ pub async fn start_enrollment(
     debug!("Transaction committed.");
 
     info!(
-        "The enrollment process for {} has ended with success.",
+        "User {} created enrollment token for user {username}.",
         session.user.username
     );
     debug!(
@@ -443,6 +453,10 @@ pub async fn start_enrollment(
         enrollment_token,
         config.enrollment_url.to_string()
     );
+    appstate.emit_event(ApiEvent {
+        context,
+        event: Box::new(ApiEventType::EnrollmentTokenAdded { user }),
+    })?;
 
     Ok(ApiResponse {
         json: json!({"enrollment_token": enrollment_token, "enrollment_url": config.enrollment_url.to_string()}),
@@ -456,11 +470,16 @@ pub async fn start_enrollment(
 ///
 /// Thanks to this endpoint you are able to receive a new desktop client configuration or update an existing one. Users need the configuration to connect to the company infrastrcture.
 ///
-/// `Enrollment token` allows to start the process of gaining access to the company infrastructure `(The enrollment token is valid for 24 hours)`. On the other hand, enrollment url allows the user to access the enrollment form via the web browser or perform the enrollment through the desktop client.
+/// `Enrollment token` allows to start the process of gaining access to the company infrastructure **(The enrollment token is valid for 24 hours)**.
+///
+/// On the other hand, enrollment url allows the user to access the enrollment form via the web browser or perform the enrollment through the desktop client.
+///
+/// Optionally this endpoint can send an email notification to the user about the enrollment.
 ///
-/// Optionally this endpoint can send an email notification to the user about the enrollment.```
 /// # Returns
-/// Returns json with `enrollment token` and `enrollment url` or `WebError` if error occurs.
+/// - JSON with `enrollment_token` and `enrollment_url`
+///
+/// - `WebError` if error occurs
 #[utoipa::path(
     post,
     path = "/api/v1/user/{username}/start_desktop",
@@ -479,6 +498,7 @@ pub async fn start_enrollment(
 )]
 pub async fn start_remote_desktop_configuration(
     session: SessionInfo,
+    context: ApiRequestContext,
     State(appstate): State<AppState>,
     Path(username): Path<String>,
     Json(data): Json<StartEnrollmentRequest>,
@@ -495,7 +515,9 @@ pub async fn start_remote_desktop_configuration(
         ));
     }
 
-    debug!("Verify that the user from the current session is an admin or only peforms desktop activation for self.");
+    debug!(
+        "Verify that the user from the current session is an admin or only peforms desktop activation for self."
+    );
     let user = user_for_admin_or_self(&appstate.pool, &session, &username).await?;
     debug!("Successfully fetched user data: {user:?}");
 
@@ -505,7 +527,9 @@ pub async fn start_remote_desktop_configuration(
         None => user.email.clone(),
     };
 
-    debug!("Create a new database transaction to save a desktop configuration token into the database.");
+    debug!(
+        "Create a new database transaction to save a desktop configuration token into the database."
+    );
     let mut transaction = appstate.pool.begin().await?;
 
     debug!(
@@ -539,6 +563,10 @@ pub async fn start_remote_desktop_configuration(
         desktop_configuration_token,
         config.enrollment_url.to_string()
     );
+    appstate.emit_event(ApiEvent {
+        context,
+        event: Box::new(ApiEventType::ClientConfigurationTokenAdded { user }),
+    })?;
 
     Ok(ApiResponse {
         json: json!({"enrollment_token": desktop_configuration_token, "enrollment_url":  config.enrollment_url.to_string()}),
@@ -552,9 +580,11 @@ pub async fn start_remote_desktop_configuration(
 /// Username is unique so database returns only single user or nothing.
 ///
 /// # Returns
-/// Returns only status code 200 if user is available or `WebError` if error occurs.
+/// - `200` if the user is available
 ///
-/// `Please take notice that if user exists in database, endpoint will return status code 400.`
+/// - `WebError` if error occurs
+///
+/// **Please take notice that if user exists in database, endpoint will return status code 400.**
 #[utoipa::path(
     post,
     path = "/api/v1/user/available",
@@ -598,16 +628,22 @@ pub async fn username_available(
 
 /// Modify user
 ///
-/// Update users data, it can remove authorized apps and active/deactivate ldap status if needed.
-/// Endpoint is able to disable a user, but `admin cannot disable himself`.
+/// Update user's data basing on `UserInfo` object, it can also remove/add authorized apps and groups assigned to user.
+///
+/// Endpoint is able to disable a user, but **admin cannot disable himself**.
+///
+/// Disabling a user can be done by setting `is_active` to `false`.
+///
 ///
 /// # Returns
-/// If erorr occurs, endpoint will return `WebError` object.
+/// - empty JSON
+///
+/// - `WebError` if error occurs
 #[utoipa::path(
     put,
     path = "/api/v1/user/{username}",
     params(
-        ("username" = String, description = "name of a user"),
+        ("username" = String, description = "Name of a user"),
     ),
     request_body = UserInfo,
     responses(
@@ -626,10 +662,14 @@ pub async fn modify_user(
     context: ApiRequestContext,
     State(appstate): State<AppState>,
     Path(username): Path<String>,
-    Json(mut user_info): Json<UserInfo>,
+    Json(user_info): Json<UserInfo>,
 ) -> ApiResult {
     debug!("User {} updating user {username}", session.user.username);
     let mut user = user_for_admin_or_self(&appstate.pool, &session, &username).await?;
+    let groups_before = UserInfo::from_user(&appstate.pool, &user).await?.groups;
+
+    // store user before mods
+    let before = user.clone();
     let old_username = user.username.clone();
     if let Err(err) = check_username(&user_info.username) {
         debug!("Username {} rejected: {err}", user_info.username);
@@ -639,6 +679,17 @@ pub async fn modify_user(
         });
     }
 
+    // check phone number
+    if let Some(ref phone) = user_info.phone {
+        if !is_valid_phone_number(phone) {
+            debug!("Invalid phone number for user {username}: {phone}");
+            return Ok(ApiResponse {
+                json: json!({}),
+                status: StatusCode::BAD_REQUEST,
+            });
+        }
+    }
+
     let status_changing = user_info.is_active != user.is_active;
 
     let mut transaction = appstate.pool.begin().await?;
@@ -687,6 +738,15 @@ pub async fn modify_user(
             user.sync_allowed_devices(&mut transaction, &appstate.wireguard_tx)
                 .await?;
         }
+
+        // remove API tokens when deactivating a user
+        if before.is_active && !user.is_active {
+            let api_tokens = ApiToken::find_by_user_id(&mut *transaction, user.id).await?;
+            for token in api_tokens {
+                token.delete(&mut *transaction).await?;
+            }
+        }
+
         user_info.into_user_all_fields(&mut user)?;
     } else {
         user_info.into_user_safe_fields(&mut user)?;
@@ -699,10 +759,10 @@ pub async fn modify_user(
         ldap_handle_user_modify(&old_username, &mut user, &appstate.pool).await;
     }
 
-    user.maybe_update_rdn().await;
+    user.maybe_update_rdn();
     user.save(&appstate.pool).await?;
 
-    ldap_update_user_state(&mut user, &appstate.pool).await;
+    Box::pin(ldap_update_user_state(&mut user, &appstate.pool)).await;
 
     if group_diff.changed() || status_changing {
         if !group_diff.added.is_empty() {
@@ -711,7 +771,7 @@ pub async fn modify_user(
                 group_diff
                     .added
                     .iter()
-                    .map(|g| g.as_str())
+                    .map(String::as_str)
                     .collect::<HashSet<&str>>(),
                 &appstate.pool,
             )
@@ -724,7 +784,7 @@ pub async fn modify_user(
                 group_diff
                     .removed
                     .iter()
-                    .map(|g| g.as_str())
+                    .map(String::as_str)
                     .collect::<HashSet<&str>>(),
                 &appstate.pool,
             )
@@ -732,29 +792,45 @@ pub async fn modify_user(
         }
     }
 
-    appstate.trigger_action(AppEvent::UserModified(user_info));
-
+    appstate.trigger_action(AppEvent::UserModified(user_info.clone()));
+    let groups_after = user_info.groups.clone();
     info!("User {} updated user {username}", session.user.username);
+
+    let set_groups_before: HashSet<_> = groups_before.iter().collect();
+    let set_groups_after: HashSet<_> = groups_after.iter().collect();
+
+    if set_groups_before != set_groups_after {
+        appstate.emit_event(ApiEvent {
+            context: context.clone(),
+            event: Box::new(ApiEventType::UserGroupsModified {
+                user: user.clone(),
+                before: groups_before,
+                after: groups_after,
+            }),
+        })?;
+    }
+
     appstate.emit_event(ApiEvent {
         context,
-        event: ApiEventType::UserModified {
-            username: user.username,
-        },
+        event: Box::new(ApiEventType::UserModified {
+            before,
+            after: user,
+        }),
     })?;
     Ok(ApiResponse::default())
 }
 
 /// Delete user
 ///
-/// Endpoint helps you delete a user, but `you can't delete yourself as a administrator`.
+/// Deletes user, however, **you can't delete yourself as an administrator**.
 ///
 /// # Returns
-/// If erorr occurs, endpoint will return `WebError` object.
+/// - `WebError` if error occurs
 #[utoipa::path(
     delete,
     path = "/api/v1/user/{username}",
     params(
-        ("username" = String, description = "name of a user"),
+        ("username" = String, description = "Name of a user"),
     ),
     responses(
         (status = 200, description = "User has been deleted."),
@@ -796,7 +872,8 @@ pub async fn delete_user(
         } else {
             None
         };
-        user.delete_and_cleanup(&mut transaction, &appstate.wireguard_tx)
+        user.clone()
+            .delete_and_cleanup(&mut transaction, &appstate.wireguard_tx)
             .await?;
 
         appstate.trigger_action(AppEvent::UserDeleted(username.clone()));
@@ -809,7 +886,7 @@ pub async fn delete_user(
         info!("User {} deleted user {}", session.user.username, &username);
         appstate.emit_event(ApiEvent {
             context,
-            event: ApiEventType::UserRemoved { username },
+            event: Box::new(ApiEventType::UserRemoved { user }),
         })?;
         Ok(ApiResponse::default())
     } else {
@@ -822,10 +899,12 @@ pub async fn delete_user(
 
 /// Change your own password
 ///
-/// Change your own password, it could return error if password is not strong enough.
+/// Changes your own password basing on `PasswordChangeSelf` object.
+///
+/// It can return error if password is not strong enough.
 ///
 /// # Returns
-/// If erorr occurs, endpoint will return `WebError` object.
+/// - `WebError` if error occurs
 #[utoipa::path(
     put,
     path = "/api/v1/user/change_password",
@@ -843,6 +922,7 @@ pub async fn delete_user(
 )]
 pub async fn change_self_password(
     session: SessionInfo,
+    context: ApiRequestContext,
     State(appstate): State<AppState>,
     Json(data): Json<PasswordChangeSelf>,
 ) -> ApiResult {
@@ -869,6 +949,10 @@ pub async fn change_self_password(
     ldap_change_password(&mut user, &data.new_password, &appstate.pool).await;
 
     info!("User {} changed his password.", &user.username);
+    appstate.emit_event(ApiEvent {
+        context,
+        event: Box::new(ApiEventType::PasswordChanged),
+    })?;
 
     Ok(ApiResponse {
         json: json!({}),
@@ -878,21 +962,23 @@ pub async fn change_self_password(
 
 /// Change user password
 ///
-/// Change user password, it could return error if password is not strong enough.
+/// Change user password basing on `PasswordChange` object, it can return error if password is not strong enough.
 ///
-/// `This endpoint doesn't allow you to change your own password. Go to: /api/v1/user/change_password.`
+/// This endpoint doesn't allow you to **change your own** password.
+///
+/// If you want to change your own password please go to: `/api/v1/user/change_password`.
 ///
 /// # Returns
-/// If erorr occurs, endpoint will return `WebError` object.
+/// - `WebError` if error occurs
 #[utoipa::path(
     put,
     path = "/api/v1/user/{username}/password",
     params(
-        ("username" = String, description = "name of a user"),
+        ("username" = String, description = "Name of a user"),
     ),
     request_body = PasswordChange,
     responses(
-        (status = 200, description = "Pasword has been changed.", body = ApiResponse, example = json!({})),
+        (status = 200, description = "Password has been changed.", body = ApiResponse, example = json!({})),
         (status = 400, description = "Bad request, password does not satisfy requirements. This endpoint does not change your own password.", body = ApiResponse, example = json!({})),
         (status = 401, description = "Unauthorized to change password.", body = ApiResponse, example = json!({"msg": "Session is required"})),
         (status = 403, description = "You don't have permission to change user password.", body = ApiResponse, example = json!({"msg": "access denied"})),
@@ -907,6 +993,7 @@ pub async fn change_self_password(
 pub async fn change_password(
     _role: AdminRole,
     session: SessionInfo,
+    context: ApiRequestContext,
     State(appstate): State<AppState>,
     Path(username): Path<String>,
     Json(data): Json<PasswordChange>,
@@ -949,6 +1036,10 @@ pub async fn change_password(
             "Admin {} changed password for user {username}",
             session.user.username
         );
+        appstate.emit_event(ApiEvent {
+            context,
+            event: Box::new(ApiEventType::PasswordChangedByAdmin { user }),
+        })?;
         Ok(ApiResponse::default())
     } else {
         debug!("Can't change password for user {username}, user not found");
@@ -961,17 +1052,17 @@ pub async fn change_password(
 
 /// Reset user password
 ///
-/// Reset user password, it will send a new enrollment to the user's email.
+/// Reset user password, it will send a new enrollment token to the user's email.
 ///
-/// `This endpoint doesn't allow you to reset your own password.`
+/// **This endpoint doesn't allow you to reset your own password.**
 ///
 /// # Returns
-/// If erorr occurs, endpoint will return `WebError` object.
+/// - `WebError` if error occurs
 #[utoipa::path(
     post,
     path = "/api/v1/user/{username}/reset_password",
     params(
-        ("username" = String, description = "name of a user"),
+        ("username" = String, description = "Name of a user"),
     ),
     responses(
         (status = 200, description = "Successfully reset user password."),
@@ -989,6 +1080,7 @@ pub async fn change_password(
 pub async fn reset_password(
     _role: AdminRole,
     session: SessionInfo,
+    context: ApiRequestContext,
     State(appstate): State<AppState>,
     Path(username): Path<String>,
 ) -> ApiResult {
@@ -1024,7 +1116,7 @@ pub async fn reset_password(
 
         let mail = Mail {
             to: user.email.clone(),
-            subject: EMAIL_PASSOWRD_RESET_START_SUBJECT.into(),
+            subject: EMAIL_PASSWORD_RESET_START_SUBJECT.into(),
             content: templates::email_password_reset_mail(
                 config.enrollment_url.clone(),
                 enrollment.id.clone().as_str(),
@@ -1058,6 +1150,10 @@ pub async fn reset_password(
             "Admin {} reset password for user {username}",
             session.user.username
         );
+        appstate.emit_event(ApiEvent {
+            context,
+            event: Box::new(ApiEventType::PasswordReset { user }),
+        })?;
         Ok(ApiResponse::default())
     } else {
         debug!("Can't reset password for user {username}, user not found");
@@ -1070,16 +1166,16 @@ pub async fn reset_password(
 
 /// Delete security key
 ///
-/// Delete Webauthn security key that allows users to authenticate.
+/// Delete WebAuthn security key that allows users to authenticate.
 ///
 /// # Returns
-/// Returns `WebError` object if error occurs.
+/// - `WebError` if error occurs
 #[utoipa::path(
     delete,
     path = "/api/v1/user/{username}/security_key/{id}",
     params(
-        ("username" = String, description = "name of a user"),
-        ("id" = i64, description = "id of security key that could point to passkey")
+        ("username" = String, description = "Name of a user"),
+        ("id" = i64, description = "ID of security key that could point to passkey")
     ),
     responses(
         (status = 200, description = "Successfully deleted security key."),
@@ -1095,6 +1191,7 @@ pub async fn reset_password(
 )]
 pub async fn delete_security_key(
     session: SessionInfo,
+    context: ApiRequestContext,
     State(appstate): State<AppState>,
     Path((username, id)): Path<(String, i64)>,
 ) -> ApiResult {
@@ -1105,12 +1202,16 @@ pub async fn delete_security_key(
     let mut user = user_for_admin_or_self(&appstate.pool, &session, &username).await?;
     if let Some(webauthn) = WebAuthn::find_by_id(&appstate.pool, id).await? {
         if webauthn.user_id == user.id {
-            webauthn.delete(&appstate.pool).await?;
+            webauthn.clone().delete(&appstate.pool).await?;
             user.verify_mfa_state(&appstate.pool).await?;
             info!(
                 "User {} deleted security key {id} for user {username}",
                 session.user.username,
             );
+            appstate.emit_event(ApiEvent {
+                context,
+                event: Box::new(ApiEventType::MfaSecurityKeyRemoved { key: webauthn }),
+            })?;
             Ok(ApiResponse::default())
         } else {
             error!(
@@ -1130,36 +1231,40 @@ pub async fn delete_security_key(
 
 /// Returns your data
 ///
-/// Endpoint returns the data associated with the current session user```
+/// Endpoint returns the data associated with the current session user
 ///
 /// # Returns
-/// Returns `UserInfo` object or `WebError` object if error occurs.
+/// - `UserInfo` object
+///
+/// - `WebError` if error occurs
 #[utoipa::path(
     get,
     path = "/api/v1/me",
     responses(
         (status = 200, description = "Returns your own data.", body = UserInfo, example = json!(
             {
-                "authorized_apps": [],
-                "email": "name@email.com",
-                "email_mfa_enabled": false,
-                "enrolled": true,
-                "first_name": "first_name",
-                "groups": [
-                    "group"
-                ],
-                "id": 1,
-                "is_active": true,
-                "last_name": "last_name",
-                "mfa_enabled": false,
-                "mfa_method": "None",
-                "phone": null,
-                "totp_enabled": false,
-                "username": "username"
-            }
+                  "authorized_apps": [],
+                  "email": "mail@mail",
+                  "email_mfa_enabled": false,
+                  "enrolled": true,
+                  "first_name": "first_name",
+                  "groups": [
+                    "admin"
+                  ],
+                  "id": 1,
+                  "is_active": true,
+                  "is_admin": true,
+                  "last_name": "last_name",
+                  "ldap_pass_requires_change": false,
+                  "mfa_enabled": false,
+                  "mfa_method": "None",
+                  "phone": 000_000_000,
+                  "totp_enabled": false,
+                  "username": "username"
+                }
         )),
         (status = 401, description = "Unauthorized return own user data.", body = ApiResponse, example = json!({"msg": "Session is required"})),
-        (status = 500, description = "Cannot retrive own user data.", body = ApiResponse, example = json!({"msg": "Internal server error"}))
+        (status = 500, description = "Cannot retrieve own user data.", body = ApiResponse, example = json!({"msg": "Internal server error"}))
     ),
     security(
         ("cookie" = []),
@@ -1174,17 +1279,17 @@ pub async fn me(session: SessionInfo, State(appstate): State<AppState>) -> ApiRe
     })
 }
 
-/// Delete Oauth token.
+/// Delete OAuth token.
 ///
-/// Endpoint helps your to delete authorized application by `OAuth2` id.
+/// Deletes an authorized application by `OAuth2` ID.
 ///
 /// # Returns
-/// Returns `WebError` object if error occurs.
+/// - `WebError` if error occurs
 #[utoipa::path(
     delete,
     path = "/api/v1/user/{username}/oauth_app/{oauth2client_id}",
     params(
-        ("username" = String, description = "name of a user"),
+        ("username" = String, description = "Name of a user"),
         ("oauth2client_id" = i64, description = "id of OAuth2 client")
     ),
     responses(
