@@ -2,15 +2,15 @@ use defguard_core::events::GrpcEvent;
 use defguard_event_logger::message::{LoggerEvent, VpnEvent};
 use tracing::debug;
 
-use crate::{error::EventRouterError, EventRouter};
+use crate::{EventRouter, error::EventRouterError};
 
 impl EventRouter {
     pub(crate) fn handle_grpc_event(&self, event: GrpcEvent) -> Result<(), EventRouterError> {
         debug!("Processing gRPC server event: {event:?}");
 
         match event {
-            GrpcEvent::GatewayConnected => todo!(),
-            GrpcEvent::GatewayDisconnected => todo!(),
+            GrpcEvent::GatewayConnected { location: _ } => todo!(),
+            GrpcEvent::GatewayDisconnected { location: _ } => todo!(),
             GrpcEvent::ClientConnected {
                 context,
                 location,
@@ -18,7 +18,7 @@ impl EventRouter {
             } => {
                 self.log_event(
                     context.into(),
-                    LoggerEvent::Vpn(VpnEvent::ConnectedToLocation { location, device }),
+                    LoggerEvent::Vpn(Box::new(VpnEvent::ConnectedToLocation { location, device })),
                 )?;
             }
             GrpcEvent::ClientDisconnected {
@@ -28,7 +28,10 @@ impl EventRouter {
             } => {
                 self.log_event(
                     context.into(),
-                    LoggerEvent::Vpn(VpnEvent::DisconnectedFromLocation { location, device }),
+                    LoggerEvent::Vpn(Box::new(VpnEvent::DisconnectedFromLocation {
+                        location,
+                        device,
+                    })),
                 )?;
             }
         }
