@@ -6,12 +6,12 @@ export const selectPasswordReset = async (page: Page) => {
 };
 
 export const setEmail = async (token: string, page: Page) => {
-  await page.getByTestId('field-email').type(token);
+  await page.getByTestId('field-email').fill(token);
   await page.getByTestId('password-reset-email-submit-button').click();
 };
 
 export const setPassword = async (password: string, page: Page) => {
-  await page.getByTestId('field-password').type(password);
-  await page.getByTestId('field-repeat').type(password);
+  await page.getByTestId('field-password').fill(password);
+  await page.getByTestId('field-repeat').fill(password);
   await page.getByTestId('password-reset-submit').click();
 };
