@@ -8,10 +8,10 @@ export const changePassword = async (page: Page, currentPassword: string) => {
   await page.getByTestId('button-change-password').click();
   const formElement = page.getByTestId('change-self-password-form');
   await formElement.waitFor({ state: 'visible' });
-  await formElement.getByTestId('field-old_password').type(currentPassword);
+  await formElement.getByTestId('field-old_password').fill(currentPassword);
   const newPassword = 'Test1234#$%';
-  await formElement.getByTestId('field-new_password').type(newPassword);
-  await formElement.getByTestId('field-repeat').type(newPassword);
+  await formElement.getByTestId('field-new_password').fill(newPassword);
+  await formElement.getByTestId('field-repeat').fill(newPassword);
   await formElement.locator('button[type="submit"]').click();
   await formElement.waitFor({ state: 'hidden', timeout: 2000 });
   return newPassword;
@@ -23,8 +23,8 @@ export const changePasswordByAdmin = async (page: Page) => {
   const formElement = page.getByTestId('change-password-admin-form');
   await formElement.waitFor({ state: 'visible' });
   const newPassword = 'Test1234#$%';
-  await formElement.getByTestId('field-new_password').type(newPassword);
-  await formElement.getByTestId('field-repeat').type(newPassword);
+  await formElement.getByTestId('field-new_password').fill(newPassword);
+  await formElement.getByTestId('field-repeat').fill(newPassword);
   await formElement.locator('button[type="submit"]').click();
   await formElement.waitFor({ state: 'hidden', timeout: 2000 });
   return newPassword;
