@@ -4,7 +4,7 @@ import type { Formatters, Locales } from './i18n-types';
 
 export const initFormatters: FormattersInitializer<Locales, Formatters> = (
   //@ts-ignore
-  locale: Locales
+  locale: Locales,
 ) => {
   const formatters: Formatters = {
     // add your formatter functions here
