@@ -1,9 +1,10 @@
 import { deepmerge } from 'deepmerge-ts';
+import { PartialDeep } from 'type-fest';
 
 import en from '../en';
 import { Translation } from '../i18n-types';
 
-const translation: Translation = {
+const translation: PartialDeep<Translation> = {
   common: {
     conditions: {
       and: 'I',
@@ -76,7 +77,7 @@ Aby dowiedzieć się więcej o:
 - Zewnętrznym SSO
 - Kontrolowaniu działania klientów VPN
 
-Pełna lista funkcjonalności enterprise: [https://docs.defguard.net/enterprise/all-enteprise-features](https://docs.defguard.net/enterprise/all-enteprise-features)</br>
+Pełna lista funkcjonalności enterprise: [https://docs.defguard.net/enterprise/enterprise-features](https://docs.defguard.net/enterprise/enterprise-features)</br>
 Informacja o licencjonowaniu: [https://docs.defguard.net/enterprise/license](https://docs.defguard.net/enterprise/license)
       `,
       controls: {
@@ -108,7 +109,7 @@ Informacja o licencjonowaniu: [https://docs.defguard.net/enterprise/license](htt
     },
     deleteStandaloneDevice: {
       title: 'Usuń urządzenie sieciowe',
-      content: 'Urządzenie {name: string} zostanie usunięte.',
+      content: 'Urządzenie {name} zostanie usunięte.',
       messages: {
         success: 'Urządzenie zostało usunięte',
         error: 'Nie udało się usunąć urządzenia.',
@@ -527,7 +528,7 @@ Informacja o licencjonowaniu: [https://docs.defguard.net/enterprise/license](htt
           },
           enableEnrollment: {
             label: 'Użyj zdalnej rejestracji',
-            link: '<a href="https://docs.defguard.net/help/enrollment" target="_blank">więcej informacji tutaj</a>',
+            link: '<a href="https://docs.defguard.net/using-defguard-for-end-users/enrollment" target="_blank">więcej informacji tutaj</a>',
           },
         },
       },
@@ -597,7 +598,7 @@ Informacja o licencjonowaniu: [https://docs.defguard.net/enterprise/license](htt
     },
     helpers: {
       setupOpt: `Możesz dodać urządzenie używając naszego klienta lub samemu skonfigurwać urządzenie.`,
-      client: `Pobierz klienta defguard <a href="https://defguard.net/download" target="_blank">tutaj</a>, a następnie postępuj zgodnie z <a href="https://docs.defguard.net/help/configuring-vpn/add-new-instance" target="_blank">instrukcją</a> w celu jego konfiguracji.`,
+      client: `Pobierz klienta defguard <a href="https://defguard.net/download" target="_blank">tutaj</a>, a następnie postępuj zgodnie z <a href="https://docs.defguard.net/using-defguard-for-end-users/desktop-client/instance-configuration" target="_blank">instrukcją</a> w celu jego konfiguracji.`,
     },
 
     steps: {
@@ -624,19 +625,6 @@ Informacja o licencjonowaniu: [https://docs.defguard.net/enterprise/license](htt
         },
         infoMessage: `<p>W razie problemów możesz odwiedzić <a href="{addDevicesDocs}">dokumentacje</a>.</p>`,
       },
-      setupMethod: {
-        manual: {
-          subTitle:
-            'Dla zaawansowanych użytkowników, pobierz konfigurację i skonfiguruj VPN na własnych zasadach.',
-          link: 'Pobierz WireGuard',
-          title: 'Konfiguracja ręczna',
-        },
-        remote: {
-          title: 'Aktywacja klienta desktop',
-          link: 'Pobierz klienta Defguard',
-          subTitle: 'Prosta konfiguracja jednym tokenem.',
-        },
-      },
       configDevice: {
         title: 'Skonfiguruj urządzenie',
         messages: {
@@ -726,9 +714,9 @@ Uwaga, podane tutaj konfiguracje nie posiadają klucza prywatnego. Musisz uzupe
       password: {
         header: 'Ustawienia hasła',
         changePassword: 'Zmiana hasła',
-        ldap_change_heading: 'Wymagana aktualizacja hasła {ldapName:string}',
+        ldap_change_heading: 'Wymagana aktualizacja hasła {ldapName}',
         ldap_change_message:
-          'Defguard nie ma możliwości odczytania twojego hasła, więc nie możemy go pobrać do automatycznej synchronizacji z danymi logowania {ldapName:string}. Aby umożliwić logowanie do innych usług za pomocą {ldapName:string}, zaktualizuj swoje hasło Defguard, aby jednocześnie ustawić hasło {ldapName:string} — możesz ponownie wpisać swoje obecne hasło, jeśli chcesz. Ten krok jest konieczny, aby zapewnić spójną i bezpieczną autoryzację w obu systemach.',
+          'Defguard nie ma możliwości odczytania twojego hasła, więc nie możemy go pobrać do automatycznej synchronizacji z danymi logowania {ldapName}. Aby umożliwić logowanie do innych usług za pomocą {ldapName}, zaktualizuj swoje hasło Defguard, aby jednocześnie ustawić hasło {ldapName} — możesz ponownie wpisać swoje obecne hasło, jeśli chcesz. Ten krok jest konieczny, aby zapewnić spójną i bezpieczną autoryzację w obu systemach.',
       },
       recovery: {
         header: 'Opcje odzyskiwania danych',
@@ -882,7 +870,7 @@ Uwaga, podane tutaj konfiguracje nie posiadają klucza prywatnego. Musisz uzupe
       },
       deleteModal: {
         title: 'Usuń API Token',
-        confirmMessage: 'API token {name: string} zostanie trwale usunięty.',
+        confirmMessage: 'API token {name} zostanie trwale usunięty.',
       },
       addModal: {
         header: 'Dodaj nowy API Token',
@@ -1032,13 +1020,9 @@ Uwaga, podane tutaj konfiguracje nie posiadają klucza prywatnego. Musisz uzupe
       label: 'Gateways',
       states: {
         error: 'Błąd pobierania statusu',
-        loading: 'Pobieranie informacji',
-        partial: 'Jeden lub więcej odłączonych',
-        connected: 'Połączone',
-        disconnected: 'Brak połączenia',
       },
       messages: {
-        error: 'Błąd pobierania statusu połączeń gatway',
+        error: 'Błąd pobierania statusu połączeń gateway',
         deleteError: 'Błąd usuwania gateway',
       },
     },
@@ -1119,6 +1103,11 @@ Uwaga, podane tutaj konfiguracje nie posiadają klucza prywatnego. Musisz uzupe
           helper:
             'Jeśli ta opcja jest włączona, Defguard automatycznie tworzy nowe konta dla użytkowników, którzy logują się po raz pierwszy za pomocą zewnętrznego dostawcy OpenID. W innym przypadku konto użytkownika musi zostać najpierw utworzone przez administratora.',
         },
+        useOpenIdForMfa: {
+          label: 'Używaj zewnętrznego OpenID dla MFA klienta',
+          helper:
+            'Gdy zewnętrzny proces Multi-Factor Authentication (MFA) OpenID SSO jest włączony, użytkownicy łączący się z lokalizacjami VPN wymagającymi MFA będą musieli uwierzytelniać się przez swoją przeglądarkę używając skonfigurowanego dostawcy dla każdego połączenia. Jeśli to ustawienie jest wyłączone, MFA dla tych lokalizacji VPN będzie obsługiwane przez wewnętrzny system SSO Defguard. W takim przypadku użytkownicy muszą mieć skonfigurowane TOTP lub MFA oparte na e-mailu.',
+        },
         usernameHandling: {
           label: 'Obsługa nazw użytkowników',
           helper:
@@ -1137,7 +1126,7 @@ Uwaga, podane tutaj konfiguracje nie posiadają klucza prywatnego. Musisz uzupe
         custom: 'Niestandardowy',
         none: 'Brak',
         documentation:
-          'Przeczytaj więcej o tej funkcji w naszej [dokumentacji](https://docs.defguard.net/enterprise/all-enteprise-features/external-openid-providers).',
+          'Przeczytaj więcej o tej funkcji w naszej [dokumentacji](https://docs.defguard.net/enterprise/enterprise-features).',
         delete: 'Usuń dostawcę',
         directory_sync_settings: {
           title: 'Ustawienia synchronizacji katalogu',
@@ -1233,6 +1222,11 @@ Uwaga, podane tutaj konfiguracje nie posiadają klucza prywatnego. Musisz uzupe
             helper:
               'Klucz prywatny dla aplikacji synchronizacji Okta w formacie JWK. Klucz nie jest wyświetlany ponownie po wgraniu.',
           },
+          jumpcloud_api_key: {
+            label: 'Klucz API JumpCloud',
+            helper:
+              'Klucz API JumpCloud używany do synchronizacji stanu użytkowników i grup.',
+          },
           group_match: {
             label: 'Synchronizuj tylko pasujące grupy',
             helper:
@@ -1333,7 +1327,6 @@ Uwaga, podane tutaj konfiguracje nie posiadają klucza prywatnego. Musisz uzupe
       },
       licenseInfo: {
         title: 'Informacje o licencji',
-        noLicense: 'Brak ważnej licencji',
         licenseNotRequired:
           "<p>Posiadasz dostęp do tej funkcji enterprise, ponieważ nie przekroczyłeś jeszcze żadnych limitów. Sprawdź <a href='https://docs.defguard.net/enterprise/license'>dokumentację</a>, aby uzyskać więcej informacji.</p>",
         types: {
@@ -1738,10 +1731,8 @@ Uwaga, podane tutaj konfiguracje nie posiadają klucza prywatnego. Musisz uzupe
     },
     stats: {
       currentlyActiveUsers: 'Obecnie aktywni użytkownicy',
-      currentlyActiveDevices: 'Obecnie aktywne urządzenia',
       activeUsersFilter: 'Aktywni użytkownicy w {hour}H',
       activeDevicesFilter: 'Aktywne urządzenia w {hour}H',
-      totalTransfer: 'Całkowity transfer:',
       activityIn: 'Aktywność w {hour}H',
       in: 'Przychodzący:',
       out: 'Wychodzący:',
@@ -1785,6 +1776,8 @@ Uwaga, podane tutaj konfiguracje nie posiadają klucza prywatnego. Musisz uzupe
       helpers: {
         address:
           'Na podstawie tego adresu będzie stworzona sieć VPN, np. 10.10.10.1/24 (sieć VPN: 10.10.10.0/24). Opcjonalnie możesz podać wiele adresów, oddzielając je przecinkiem. Pierwszy adres będzie adresem głównym i zostanie użyty do przypisywania adresów IP urządzeniom. Pozostałe adresy są dodatkowe i nie będą zarządzane przez Defguarda.',
+        endpoint:
+          'Publiczny adres IP lub domena internetowa, do której będą łączyć się użytkownicy/urządzenia. Ten adres zostanie użyty w konfiguracji klientów, ale Gatewaye Defguard nie wiążą się z tym adresem.',
         gateway:
           'Adres publiczny Gatewaya, używany przez użytkowników VPN do łączenia się.',
         dns: 'Określ resolwery DNS, które mają odpytywać, gdy interfejs WireGuard jest aktywny.',
@@ -1804,7 +1797,7 @@ Uwaga, podane tutaj konfiguracje nie posiadają klucza prywatnego. Musisz uzupe
           label: 'Adres i maska sieci VPN',
         },
         endpoint: {
-          label: 'Adres gatewaya',
+          label: 'Adres IP lub domena internetowa Gatewaya',
         },
         allowedIps: {
           label: 'Dozwolone adresy IP',
@@ -1868,7 +1861,7 @@ Uwaga, podane tutaj konfiguracje nie posiadają klucza prywatnego. Musisz uzupe
       noConnection: `Brak połączenia proszę uruchom poniższą komendę.`,
       connected: `Gateway połączony.`,
       statusError: 'Nie udało się uzyskać statusu',
-      oneLineInstall: `Jeśli wykonujesz instalację w jednej linii: https://docs.defguard.net/admin-and-features/setting-up-your-instance/one-line-install
+      oneLineInstall: `Jeśli wykonujesz instalację w jednej linii: https://docs.defguard.net/getting-started/one-line-install
         nie ma potrzeby wykonywania dalszych kroków.`,
       fromPackage: `Zainstaluj pakiet dostępny na https://github.com/DefGuard/gateway/releases/latest i skonfiguruj \`/etc/defguard/gateway.toml\`
         na podstawie [dokumentacji]({setupGatewayDocs}).`,
@@ -2110,7 +2103,6 @@ W przypadku innych zgłoszeń skontaktuj się z nami: support@defguard.net
         labels: {
           name: 'Nazwa',
           location: 'Położenie',
-          assignedIp: 'Adres IP',
           description: 'Opis',
           addedBy: 'Dodane przez',
           addedAt: 'Data dodania',
@@ -2181,7 +2173,7 @@ W przypadku innych zgłoszeń skontaktuj się z nami: support@defguard.net
         modals: {
           applyConfirm: {
             title: 'Wdróż oczekujące zmiany',
-            subtitle: '{count: number} zmian zostanie zastosowanych',
+            subtitle: '{count} zmian zostanie zastosowanych',
             submit: 'Wdróż zmiany',
           },
           filterGroupsModal: {
@@ -2199,12 +2191,12 @@ W przypadku innych zgłoszeń skontaktuj się z nami: support@defguard.net
           addNew: 'Dodaj nową',
           filter: {
             nothingApplied: 'Filtr',
-            applied: 'Filtry ({count: number})',
+            applied: 'Filtry ({count})',
           },
           apply: {
             noChanges: 'Wdróż oczekujące zmiany',
-            all: 'Wdróż oczekujące zmiany ({count: number})',
-            selective: 'Wdróż zaznaczone zmiany ({count: number})',
+            all: 'Wdróż oczekujące zmiany ({count})',
+            selective: 'Wdróż zaznaczone zmiany ({count})',
           },
         },
         list: {
@@ -2257,7 +2249,7 @@ W przypadku innych zgłoszeń skontaktuj się z nami: support@defguard.net
             title: 'Usuwanie zablokowane',
             //md
             content: `
-Ten alias jest obecnie używany przez nastąpujące reguły i nie może być usunięty. Aby go usunąć, należy najpierw wykasować go z tych reguł({rulesCount: number}):
+Ten alias jest obecnie używany przez nastąpujące reguły i nie może być usunięty. Aby go usunąć, należy najpierw wykasować go z tych reguł({rulesCount}):
 `,
           },
           filterGroupsModal: {
@@ -2299,12 +2291,12 @@ Ten alias jest obecnie używany przez nastąpujące reguły i nie może być usu
           addNew: 'Dodaj nową',
           filter: {
             nothingApplied: 'Filtr',
-            applied: 'Filtry ({count: number})',
+            applied: 'Filtry ({count})',
           },
           apply: {
             noChanges: 'Wdróż oczkujące zmiany',
-            all: 'Wdróż oczkujące zmiany ({count: number})',
-            selective: 'Wdróż zaznaczone zmiany ({count: number})',
+            all: 'Wdróż oczkujące zmiany ({count})',
+            selective: 'Wdróż zaznaczone zmiany ({count})',
           },
         },
         list: {
@@ -2395,8 +2387,8 @@ Ten alias jest obecnie używany przez nastąpujące reguły i nie może być usu
       },
     },
   },
-};
+} as PartialDeep<Translation>;
 
-const pl = deepmerge(en, translation);
+const pl = deepmerge(en, translation) as Translation;
 
 export default pl;
