@@ -2,7 +2,7 @@ import { omit } from 'lodash-es';
 import { createJSONStorage, persist } from 'zustand/middleware';
 import { createWithEqualityFn } from 'zustand/traditional';
 
-import { OverviewLayoutType, OverviewStore } from '../../../../shared/types';
+import { OverviewLayoutType, type OverviewStore } from '../../../../shared/types';
 
 export const useOverviewStore = createWithEqualityFn<
   OverviewStore,
@@ -14,7 +14,6 @@ export const useOverviewStore = createWithEqualityFn<
       networks: [],
       viewMode: OverviewLayoutType.GRID,
       defaultViewMode: OverviewLayoutType.GRID,
-      statsFilter: 1,
       setState: (newValues) => set((state) => ({ ...state, ...newValues })),
     }),
     {
