@@ -1,8 +1,7 @@
 import './style.scss';
 
-import { shallow } from 'zustand/shallow';
-
 import { useI18nContext } from '../../../../i18n/i18n-react';
+import { isPresent } from '../../../../shared/defguard-ui/utils/isPresent';
 import { useUserProfileStore } from '../../../../shared/hooks/store/useUserProfileStore';
 import { AddComponentBox } from '../../shared/components/AddComponentBox/AddComponentBox';
 import { useAddApiTokenModal } from '../../shared/modals/AddApiTokenModal/useAddApiTokenModal';
@@ -13,7 +12,7 @@ import { DeleteApiTokenModal } from './DeleteApiTokenModal/DeleteApiTokenModal';
 export const UserApiTokens = () => {
   const { LL } = useI18nContext();
   const user = useUserProfileStore((state) => state.userProfile?.user);
-  const openAddApiTokenModal = useAddApiTokenModal((s) => s.open, shallow);
+  const openAddApiTokenModal = useAddApiTokenModal((s) => s.open);
 
   return (
     <section id="user-api-tokens">
@@ -21,9 +20,10 @@ export const UserApiTokens = () => {
         <h2>{LL.userPage.apiTokens.header()}</h2>
       </header>
       <ApiTokenList />
-      {user && (
+      {isPresent(user) && (
         <AddComponentBox
           data-testid="add-api-token-button"
+          text={LL.userPage.apiTokens.addToken()}
           callback={() => {
             if (user) {
               openAddApiTokenModal({
@@ -31,7 +31,6 @@ export const UserApiTokens = () => {
               });
             }
           }}
-          text={LL.userPage.apiTokens.addToken()}
         />
       )}
       <DeleteApiTokenModal />
