@@ -1,7 +1,7 @@
 import { z } from 'zod';
 
 export const versionUpdateToastMetaSchema = z.object({
-  customId: z.string().min(1),
+  customId: z.string().trim().min(1),
 });
 
 export type VersionUpdateToastMeta = z.infer<typeof versionUpdateToastMetaSchema>;
