@@ -1,4 +1,4 @@
-import { PropsWithChildren, useEffect } from 'react';
+import { type PropsWithChildren, useEffect } from 'react';
 import { shallow } from 'zustand/shallow';
 
 import { useI18nContext } from '../../../i18n/i18n-react';
@@ -10,26 +10,26 @@ const ApiContextManager = ({ children }: PropsWithChildren) => {
 
   const { LL } = useI18nContext();
 
+  // biome-ignore lint/correctness/useExhaustiveDependencies: migration, checkMeLater
   useEffect(() => {
     if (client && LL && LL.messages) {
       const defaultResponseInterceptor = client.interceptors.response.use(
         (res) => {
-          // API sometimes returns null in optional fields.
+          // API returns null in optional fields.
           if (res.data) {
-            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
             res.data = removeNulls(res.data);
           }
           return res;
         },
         (error) => {
           console.error('Axios Error ', error);
+          throw error;
         },
       );
       return () => {
         client.interceptors.response.eject(defaultResponseInterceptor);
       };
     }
-    // eslint-disable-next-line react-hooks/exhaustive-deps
   }, [LL?.messages, client]);
 
   if (!client || !endpoints) return null;
