@@ -1,6 +1,6 @@
 import { createWithEqualityFn } from 'zustand/traditional';
 
-import { OpenidClientStore } from '../../types';
+import type { OpenidClientStore } from '../../types';
 
 export const useOpenidClientStore = createWithEqualityFn<OpenidClientStore>(
   (set) => ({
